<?php
namespace Teknisa\MAN\Controllers;

use Teknisa\MAN\Services\ExpImpNfceService;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\MAN\Utilities\Resposta;
use Teknisa\MAN\Utilities;
use \Datetime;
use Zeedhi\Framework\DTO\Response\Message;

use NFePHP\NFe\Common\Standardize;
use NFePHP\Common\Certificate;
use NFePHP\Common\Soap\SoapCurl;
use NFePHP\NFe\Make;
use NFePHP\NFe\Tools;

use \Exception as Exception;

use Teknisa\MAN\Services\TransmissaoNfceNewService;
use Teknisa\MAN\Factories\TransmissaoNFCE;
use Zeedhi\Framework\DependencyInjection\InstanceManager;


class ExpImpNfceController {     
    
   
    private $expImpNfceService, $utilService,  $nfeTools, $transmissaoService, $notaFiscal,$entityManager;
    private $url = "https://rm.teknisa.com/export-xml-nfce/";
    const VERSAO_LAYOUT = '4.00';
    
    
    public static $logMessages = array();
    
    public function __construct (Utilities\Util $utilService, ExpImpNfceService $expImpNfceService, $instanceManager, TransmissaoNfceNewService $transmissaoService, TransmissaoNFCE $notaFiscal,\Doctrine\ORM\EntityManager $entityManager){
        $this->expImpNfceService = $expImpNfceService;
        $this->utilService = $utilService;
        $this->instanceManager = $instanceManager;
        $this->transmissaoService = $transmissaoService;
        $this->notaFiscal = $notaFiscal;
        $this->entityManager = $entityManager;
    }
    
    
    public function findLog(Request\Filter $request, Response $response) {
        
        ini_set('max_execution_time', 1800);
		ini_set('memory_limit', '1G');
		
        $parametros = $this->buildParameters($request);
        $result = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_CONSULTA_LOGARQ, $parametros);
        if ($result->error())
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        else 
            $response->addDataSet(new DataSet('vnd04400_expimpxmlnfce_logArquivos', $result->getResult()));
    
    }
    
    public function findRelErroAlerta(Request\Filter $request, Response $response) {
        
        ini_set('max_execution_time', 1800);
		ini_set('memory_limit', '1G');
		
        $parametros = $this->buildParameters($request);
        $result = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_CONSULTA_ERRO_ALERTA, $parametros);
        
        if ($result->error())
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        else 
            $response->addDataSet(new DataSet('vnd04400_expimpxmlnfce_RelErroAlerta', $result->getResult()));
    
    }
    
    public function findLogMsg(Request\Filter $request, Response $response) {
        
        ini_set('max_execution_time', 1800);
		ini_set('memory_limit', '1G');
		
        $parametros = $this->buildParameters($request);
        $result = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_CONSULTA_LOGMSG, $parametros);
        
        if ($result->error())
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        else 
            $response->addDataSet(new DataSet('vnd04400_expimpxmlnfce_logMensagens', $result->getResult()));
    
    }
    
   
    
    private function setVendas ($arrayDados,$conn){
                $cdsVendaXml         =  $arrayDados['cdsVendaXml'];     
                $cdsItensXml         =  $arrayDados['cdsItensXml'];     
                $cdsMeioPgtoXml      =  $arrayDados['cdsMeioPgtoXml'];  
                $stFilial            =  $arrayDados['stFilial'];            
                $stCdCaixa           =  $arrayDados['stCdCaixa'];           
                $stEmpresa           =  $arrayDados['stEmpresa'];           
                $stNomeXml           =  $arrayDados['seFiles'];             
                $stCdOperador        =  $arrayDados['stCdOperador'];
                $stTipoEmissao       =  $arrayDados['stTipoEmissao'];
              $arrayDados['error']['NRNOTAFISCALCE']   = $cdsVendaXml['NRNOTAFISCALCE'];
              //Verifica se a venda já existe no banco
                $params['CDFILIAL']       =   $stFilial;
                $params['NRNOTAFISCALCE'] =   $cdsVendaXml['NRNOTAFISCALCE'];
                $params['DTENTRVENDA']    =   substr($cdsVendaXml['DTVENDA'],0,10);
                
                if($arrayDados['isSat']){ 
                    $params['CDSERIESAT'] = $cdsVendaXml['CDSERIESAT'];
                    $consultaVenda = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIF_VENDA_SAT,$params,$conn)->getResult();
                }else{
                    $params['CDSERIENFCE']  = $cdsVendaXml['CDSERIENFCE'];
                    $consultaVenda = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIF_VENDA_NFCE,$params,$conn)->getResult();
                } 
                 
                 
                 
                if(empty($consultaVenda)){
                    
                    $params['NRACESSONFCE'] = $cdsVendaXml['NRACESSONFCE'];
                    //Verifica venda existente pela chave de acesso
                    $consultaVendaChaveAcesso = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIF_VENDA_NRACESSONFCE,$params,$conn)->getResult();
                    
                    if(!empty($consultaVendaChaveAcesso)){
                        $arrayDados = $this->buildRetorno($arrayDados,'T','E','Venda com a chave de acesso já existe no sistema.','','','','');
                    
                        $this->expImpNfceService->insertMsgLogArquivoXml($arrayDados,$conn);
                            
                        return $arrayDados;
                    }
                    
                    // Verifica o caixa
                     if(empty($stCdCaixa)){
                          $stCdCaixa = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIF_CAIXA,$params,$conn)->getResult();
                      }
                    
                    
                    //Verifica se o consumidor da Venda está cadastrado no banco 
                    $params['NRINSCRCONS'] = $cdsVendaXml['NRINSCRCONS'];
                    
                    $consultaConsumidor = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIF_CONSUMIDOR,$params,$conn)->getResult();
                    
                    if(empty($consultaConsumidor[0]['CDCLIENTE'])){
                        $consultaConsumidorPadrao = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_GET_CONSUMIDOR_PADRAO,$params,$conn)->getResult();
                        
                        if(!empty($consultaConsumidorPadrao[0]['CDCLIENTE']) && !empty($consultaConsumidorPadrao[0]['CDCCUSCLIE'])){
                            $consumidorPadrao = true;
                            $params['CDCLIENTE']    =   $consultaConsumidorPadrao[0]['CDCLIENTE'];
                            $params['CDCCUSCLIE']   =   $consultaConsumidorPadrao[0]['CDCCUSCLIE'];
                            $params['NMCONSVEND']   =   $consultaConsumidorPadrao[0]['NMCONSUMIDOR'];
                            $params['CDCONSUMIDOR'] =   $consultaConsumidorPadrao[0]['CDCONSUMIDOR'];
                        }else { 
                                
                                $arrayDados = $this->buildRetorno($arrayDados,'T','E','Filial não possui cliente e centro de custo padrão cadastrados.','Verifique o cadastro na tela Parametrização > Vendas > Cliente.','','','');
                                $this->expImpNfceService->insertMsgLogArquivoXml($arrayDados,$conn);
                                return $arrayDados;
                        }
                    }else {
                        $consumidorPadrao = false;
                        $params['CDCLIENTE']    =   $consultaConsumidor[0]['CDCLIENTE'];
                        $params['CDCCUSCLIE']   =   $consultaConsumidor[0]['CDCCUSCLIE'];
                        $params['NMCONSVEND']   =   $consultaConsumidor[0]['NMCONSUMIDOR'];
                        $params['CDCONSUMIDOR'] =   $consultaConsumidor[0]['CDCONSUMIDOR'];
                    }
                    
                    //Verifica Abertura
                    $params['CDCAIXA'] = $stCdCaixa;
                    $params['DTVENDA'] = $cdsVendaXml['DTVENDA'];
                    $consultaAbertura = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIFICA_ABERTURA,$params,$conn)->getResult();
                    
                    if(empty($consultaAbertura)){
                        $params['DTABERCAIX']   = substr(strval($cdsVendaXml['DTVENDA']),0,10) . ' 00:00:01';
                        $params['DTFECHCAIX']   = substr(strval($cdsVendaXml['DTVENDA']),0,10) . ' 23:59:59';
                        $params['DTMOVTURCAIX'] = substr(strval($cdsVendaXml['DTVENDA']),0,10);
                        $params['CDOPERADOR']   = $stCdOperador; 
                        $params['DSOBSCAIXA']   = 'VIA XML '.($arrayDados['isSat'] ? 'SAT' : 'NFCE');
                        
                        $result = $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::QUERY_INSERT_TURCAIXA,$params,$conn);
                        $stDtAberCaixa = substr(strval($cdsVendaXml['DTVENDA']),0,10) . ' 00:00:01';
                       
                    }else{
                        $stDtAberCaixa = $consultaAbertura[0]['DTABERCAIX'];
                        }
                       
                    //Verifica Loja
                    $consultaLoja = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIFICA_LOJA,$params,$conn)->getResult();
                   
                    //Insere a Venda no Banco
                    $nrSeqVenda = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_GERA_NRSEQVENDA,$params,$conn)->getResult();
                    
                    $params['NRSEQVENDA']       = $nrSeqVenda[0]['NRSEQVENDA'];
                    $params['CDEMPRESA']        = $stEmpresa;
                    $params['IDSITUVENDA']      = 'O';
                    $params['DTABERTUR']        = $stDtAberCaixa;
                    $params['DTFECHAVENDA']     = $cdsVendaXml['DTVENDA'];
                    $params['CDLOJA']           = $consultaLoja[0]['CDLOJA'];
                    $params['DTABERVENDA']      = $cdsVendaXml['DTVENDA'];
                    $params['CDSERIESAT']       = $cdsVendaXml['CDSERIESAT'];
                    $params['CDSERIENFCE']      = $cdsVendaXml['CDSERIENFCE'];
                    $params['NRACESSONFCE']     = $cdsVendaXml['NRACESSONFCE'];
                    $params['NRNOTAFISCALCE']   = $cdsVendaXml['NRNOTAFISCALCE'];
                    $params['NRLANCTONFCE']     = $cdsVendaXml['NRLANCTONFCE'];
                    $params['IDTPEMISVEND']     = $arrayDados['isSat'] ? 'S' : 'N';
                    $params['IDTPAMBNFCE']      = $cdsVendaXml['IDTPAMBNFCE'] ? $cdsVendaXml['IDTPAMBNFCE'] : '1';
                    $params['NRINSJURIEST']     = $cdsVendaXml['NRINSJURFILI'];
                    
                    $params['DTHRPROTOCONFCE']  = $arrayDados['isSat'] ? '' : $cdsVendaXml['DTHRPROTOCONFCE'];
                    $params['NRPROTOCOLONFCE']  = $arrayDados['isSat'] ? '' : $cdsVendaXml['NRPROTOCOLONFCE'];
                    $params['DSOBSSTATUSNFCE']  = $arrayDados['isSat'] ? '' : $cdsVendaXml['DSOBSSTATUSNFCE'];
                    
                    $params['SGESTADO']         = $cdsVendaXml['SGESTADO'];
                    $params['DSARQXMLNFCE']     = $cdsVendaXml['DSARQXMLNFCE'];
                  
                   //Executa Insert da VENDA
                   
                    $verificaGorjeta  = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_GORJETA_ITEMVENDA,$params,$conn)->getResult();
                    $verificaDelivery  = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_DELIVERY_ITEMVENDA,$params,$conn)->getResult();
                    
                    if ($verificaGorjeta[0]['IDTRATTAXASERV'] == 'P' && ($verificaGorjeta[0]['IDCOMISVENDA'] == 'S' || $verificaGorjeta[0]['IDCOMISVENDA'] == 'G')) {
                        $aux = $verificaGorjeta[0]['CDPRODTAXASERV'];
                        $resultInsertVenda = $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::QUERY_INSERT_VENDA_MESA,$params,$conn);
                        
                    } else if ($verificaDelivery[0]['IDTRATTAXAENTR'] == 'P' && ($verificaDelivery[0]['IDCOMISVENDA'] == 'S' || $verificaDelivery[0]['IDCOMISVENDA'] == 'G')) {
                        $aux = $verificaDelivery[0]['CDPRODTAXAENTR'];
                        $resultInsertVenda = $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::QUERY_INSERT_VENDA_DELIVERY,$params,$conn);
                        
                    } else {
                        $aux = 'ZZZZZZZ';
                        $resultInsertVenda = $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::QUERY_INSERT_VENDA,$params,$conn);
                    }

                    
                    if($resultInsertVenda->error()){
                        $arrayDados = $this->buildRetorno($arrayDados,'T','E','Erro ao  inserir dados na tabela de venda','','','','');
                        $this->expImpNfceService->insertMsgLogArquivoXml($arrayDados,$conn);
                        return $arrayDados;
                    }
                    
                    
                    //Insere arquivo XML no banco
                          $paramsArqXml['CDFILIAL']     = $stFilial;
                          $paramsArqXml['CDCAIXA']      = $stCdCaixa;
                          $paramsArqXml['NRSEQVENDA']   = $nrSeqVenda[0]['NRSEQVENDA'];
                          $paramsArqXml['DSARQXMLNFCE'] = $cdsVendaXml['DSARQXMLNFCE'];
                            
                    $resultInsertArqXml = $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::QUERY_INSERT_ARQXML,$paramsArqXml,$conn);
                    if($resultInsertArqXml->error()){
                        $arrayDados = $this->buildRetorno($arrayDados,'T','E','Erro ao  inserir dados na tabela de arquivo xml','','','','');
                        $this->expImpNfceService->insertMsgLogArquivoXml($arrayDados,$conn);
                        return $arrayDados;
                    }
                    
                    //Insere o Item da Venda no Banco
                    
                        for ($i = 0; $i <= count($cdsItensXml['CDPRODUTO'])- 1; $i++){
                           
                          //Verifica se o produto existe no sistema
                          $params['CDBARPRODUTO'] = $cdsItensXml['CDPRODUTO'][$i];
                          $consultaCdBarProduto =  $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIFICA_CODBAR,$params,$conn)->getResult();
                            
                          if(empty($consultaCdBarProduto)){
                            $params['CDBARRAPROD'] =  $cdsItensXml['CDPRODUTO'][$i];
                            $consultaCdBarProdTab =  $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIF_CODBAR_PROD_TAB,$params,$conn)->getResult();
                            
                                if(empty($consultaCdBarProdTab)){
                                    $stCdProduto = '';
                                }else{
                                    $stCdProduto   = $consultaCdBarProdTab[0]['CDARVPROD'];
                                    $stNomeProduto = $consultaCdBarProdTab[0]['NMPRODUTO'];
                                }
                          }else{
                              $stCdArvProduto = $consultaCdBarProduto[0]['CDARVPROD'];
                              $stCdProduto = $consultaCdBarProduto[0]['CDPRODUTO'];
                              $stNomeProduto =  $consultaCdBarProduto[0]['NMPRODUTO'];
                          }
                          
                          //vrTotItem calculado antes para apos a alteração de acressimo e valor unitario do produto 	
                          $vrTotItem = (floatval($cdsItensXml['QTPRODVEND'][$i]) * floatval($cdsItensXml['VRUNITVEND'][$i])) -	
                                       (floatval($cdsItensXml['VRDESITVEND'][$i]) ? floatval($cdsItensXml['VRDESITVEND'][$i]) : 0) +	
                                       (floatval($cdsItensXml['VRACRITVEND'][$i]) ? floatval($cdsItensXml['VRACRITVEND'][$i]) : 0);
                           
                          if(empty($stCdProduto)){
                                $arrayDados = $this->buildRetorno($arrayDados,'T','E','Produto não cadastrado no banco de dados.','Verifique o produto no arquivo XML.',$cdsItensXml['CDPRODUTO'][$i],substr($cdsItensXml['NMPRODUTO'][$i],0,50),'');
                                $this->expImpNfceService->insertMsgLogArquivoXml($arrayDados,$conn);
                                return $arrayDados;
                          }
                          
                          // PONTO A SER ALTERADO 
                          //verifica aliquiota do produtos
                          $params['CDPRODUTO'] = $stCdProduto;
                          $consultaAliq = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIF_ALIQ,$params,$conn)->getResult();
                          
                          $validaImposto = false;
                          if(empty($consultaAliq)){
                                $arrayDados = $this->buildRetorno($arrayDados,'T','E','Produto "'.$stCdArvProduto.'" não possui imposto cadastrado no banco de dados.','Verifique o cadastro na tela Parametrização > Vendas > Produto > Imposto do Produto.',$stCdArvProduto,$stNomeProduto,'');
                                $this->expImpNfceService->insertMsgLogArquivoXml($arrayDados,$conn);
                                return $arrayDados;
                          }else{
                              $validaImposto = true;
                              $error   = false;
                              $stError = 'Existem pendências no cadastro de imposto do produto: '. $stCdArvProduto;
                              if(is_null($consultaAliq[0]['CDIMPOSTO'])){
                                  $error   = true;
                                  $stError = $stError . ', Imposto, ';
                              }
                              if(is_null($consultaAliq[0]['VRPEALIMPFIS'])){
                                  $error   = true;
                                  $stError = $stError . ' Alíquota, ';
                              }
                              if(is_null($consultaAliq[0]['CDCFOPPFIS'])){
                                  $error   = true;
                                  $stError = $stError . ' Código CFOP, ';
                              }
                              if(is_null($consultaAliq[0]['CDCSTICMS'])){
                                  $error   = true;
                                  $stError = $stError . ' Código CST (ICMS), ';
                              }
                              if(is_null($consultaAliq[0]['CDCSTPISCOF'])){
                                  $error   = true;
                                  $stError = $stError . ' Código CST (PIS/COFINS), ';
                              }
                              if(is_null($consultaAliq[0]['VRALIQPIS'])){
                                  $error   = true;
                                  $stError = $stError . ' Alíquota PIS, ';
                              }
                              if(is_null($consultaAliq[0]['VRALIQCOFINS'])){
                                  $error   = true;
                                  $stError = $stError . ' Alíquota COFINS. ';
                              }
                              if($error){
                                  
                                $arrayDados = $this->buildRetorno($arrayDados,'T','E',$stError,'Verifique o cadastro na tela Parametrização > Vendas > Produto > Imposto do Produto.',$stCdArvProduto,$stNomeProduto,'');
                                
                                $this->expImpNfceService->insertMsgLogArquivoXml($arrayDados,$conn);
                                return $arrayDados;
                                   
                              }
                          }
                          
                          if($validaImposto && isset($cdsItensXml['PRICMS']) && isset($cdsItensXml['PRPIS']) && isset($cdsItensXml['PRCOFIN'])){
                              $error   = false;
                              $stError = 'Imposto do XML difere do imposto cadastrado no sistema: '. $stCdArvProduto;
                              if(floatval($consultaAliq[0]['VRPEALIMPFIS']) != floatval($cdsItensXml['PRICMS'][$i])){
                                  $error   = true;
                                  $stError = $stError . ' Alíq. Parametrizada:'.$consultaAliq[0]['VRPEALIMPFIS'].', Aliq. do xml:'.$cdsItensXml['PRICMS'][$i];
                              }
                              if(floatval($consultaAliq[0]['VRALIQPIS']) != floatval($cdsItensXml['PRPIS'][$i])){
                                  $error   = true;
                                  $stError = $stError . ', Alíq PIS Parametrizada: '.$consultaAliq[0]['VRALIQPIS'].', Aliq. PIS XML: '. $cdsItensXml['PRPIS'][$i];
                              }
                              if(floatval($consultaAliq[0]['VRALIQCOFINS']) != floatval($cdsItensXml['PRCOFIN'][$i])){
                                  $error   = true;
                                  $stError = $stError . ', Alíquota COFINS Parametrizada: '.$consultaAliq[0]['VRALIQCOFINS'].', Aliq. COFINS XML: '. $cdsItensXml['PRCOFIN'][$i];
                              }
                              if($error){
                                  
                                $arrayDados = $this->buildRetorno($arrayDados,'T','E',$stError,'Verifique o cadastro na tela Parametrização > Vendas > Produto > Imposto do Produto.',$stCdArvProduto,$stNomeProduto,'');
                                
                                $this->expImpNfceService->insertMsgLogArquivoXml($arrayDados,$conn);
                                return $arrayDados;
                                   
                              }
                          }
                          
                          $params['NRSEQUITVEND']      = $cdsItensXml['NRSEQUITVEND'][$i];	
                          $params['QTPRODVEND']        = $cdsItensXml['QTPRODVEND'][$i];	
                          $params['VRUNITVENDCL']      = 0;	
                          $params['VRDESITVEND']       = $cdsItensXml['VRDESITVEND'][$i] ? $cdsItensXml['VRDESITVEND'][$i] : 0;	
                          if($stCdProduto === $aux){	
                            $params['VRUNITVEND']        = 0;	
                            $params['VRACRITVEND']       = (floatval($cdsItensXml['QTPRODVEND'][$i]) * floatval($cdsItensXml['VRUNITVEND'][$i])) -	
                                                                    (floatval($cdsItensXml['VRDESITVEND'][$i]) ? floatval($cdsItensXml['VRDESITVEND'][$i]) : 0) +	
                                                                    (floatval($cdsItensXml['VRACRITVEND'][$i]) ? floatval($cdsItensXml['VRACRITVEND'][$i]) : 0);	
                          }	
                          else{	
                            $params['VRUNITVEND']        = $cdsItensXml['VRUNITVEND'][$i];	
                            $params['VRACRITVEND']       = $cdsItensXml['VRACRITVEND'][$i] ? $cdsItensXml['VRACRITVEND'][$i] : 0;	
                          }
                          
                          
                          if($consultaAliq[0]['IDTPIMPOSFIS'] == 'T'){
                              $params['CDORDALIQIMPR'] = '01';
                          }else if($consultaAliq[0]['IDTPIMPOSFIS'] == 'F'){
                              $params['CDORDALIQIMPR'] = 'FF';
                          }else if($consultaAliq[0]['IDTPIMPOSFIS'] == 'I'){
                              $params['CDORDALIQIMPR'] = 'II';
                          }else if($consultaAliq[0]['IDTPIMPOSFIS'] == 'N'){
                              $params['CDORDALIQIMPR'] = 'NN';
                          }  
                           
                          $params['VRALIQIMPR']        = $consultaAliq[0]['VRPEALIMPFIS'];
                           
                          //Executa Insert itemVenda
                          $resultInsertItemVenda  = $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::QUERY_INSERT_ITEMVENDA,$params,$conn);
                           if($resultInsertItemVenda->error()){
                                $arrayDados = $this->buildRetorno($arrayDados,'T','E','Erro ao  inserir dados na tabela de itens da venda','','','','');
                                $this->expImpNfceService->insertMsgLogArquivoXml($arrayDados,$conn);
                                return $arrayDados;
                            }
                            
                        
                          //Insere ITVENDAIMPOS
                          
                         
                          $vrImposto       = $this->utilService->roundABNT($vrTotItem * floatval($consultaAliq[0]['VRPEALIMPFIS'])/100,2);
                          $vrImpPis        = $this->utilService->roundABNT($vrTotItem * floatval($consultaAliq[0]['VRALIQPIS'])/100,2); 
                          $vrImpCofins     = $this->utilService->roundABNT($vrTotItem * floatval($consultaAliq[0]['VRALIQCOFINS'])/100,2);
                          $vrBaseCalcReduz = 0;
                          $vrImpoProdReduz = 0;
                          $vrBReduzIcms    = 0;
                          
                            // adicionar um if verificando o $consultaAliq[0]['VRPERCREDUCAO'] caso esteja zerado, passar zero nas 3 variaveis	
                        if ($consultaAliq[0]['VRPERCREDUCAO'] == 0) {	
                          $vrBaseCalcReduz = 0;	
                          $vrImpoProdReduz = 0;	
                          $vrBReduzIcms    = 0;	
                        }	
                        else {	
                          $vrBaseCalcReduz = floor(($vrTotItem - ($vrTotItem * floatval($consultaAliq[0]['VRPERCREDUCAO'])/100)) * 100) / 100; // trunca o resultado em 2 casas decimais	
                          //base calculo redução ( caso o VRPERCREDUCAO esteja zerado salvar zero  )	
                          	
                          $vrImpoProdReduz = $this->utilService->roundABNT($vrBaseCalcReduz * floatval($consultaAliq[0]['VRPEALIMPFIS'])/100,2);	
                          $vrBReduzIcms    = $vrTotItem - $vrBaseCalcReduz;	
                        }
                          
                           
                          $paramsVendaImpos['CDFILIAL']              = $stFilial;
                          $paramsVendaImpos['CDCAIXA']               = $stCdCaixa;
                          $paramsVendaImpos['NRSEQVENDA']            = $nrSeqVenda[0]['NRSEQVENDA'];
                          $paramsVendaImpos['NRSEQUITVEND']          = $cdsItensXml['NRSEQUITVEND'][$i];
                          $paramsVendaImpos['NRSEQITIMPOS']          = '001';
                          $paramsVendaImpos['CDIMPOSTO']             = $consultaAliq[0]['CDIMPOSTO'];
                          $paramsVendaImpos['VRPEALPRODIT']          = floatval($consultaAliq[0]['VRPEALIMPFIS']); 
                          $paramsVendaImpos['VRIMPOPRODIT']          = floatval($vrImposto); 
                          $paramsVendaImpos['VRPERCOFINS']           = floatval($consultaAliq[0]['VRALIQCOFINS']);
                          $paramsVendaImpos['VRPERPIS']              = floatval($consultaAliq[0]['VRALIQPIS']);
                          $paramsVendaImpos['VRPRBCREDUICMS']        = floatval($consultaAliq[0]['VRPERCREDUCAO']);
                          
                          $paramsVendaImpos['VRBASECALCREDUZ']       = floatval($vrBaseCalcReduz);  //base calculo redução ( caso o VRPERCREDUCAO esteja zerado salvar zero  )
                          $paramsVendaImpos['VRIMPOPRODREDUZ']       = floatval($vrImpoProdReduz);
                          $paramsVendaImpos['VRBASECALCICMS']        = floatval($vrTotItem);
                          $paramsVendaImpos['VRBCREDUZICMS']         = floatval($vrBReduzIcms); 
                          $paramsVendaImpos['VRBCPISCOFINS']         = floatval($vrTotItem);
                          $paramsVendaImpos['VRIMPPIS']              = floatval($vrImpPis); 
                          $paramsVendaImpos['VRIMPCOFINS']           = floatval($vrImpCofins);           
                          $paramsVendaImpos['CDCSTPRODI']            = $consultaAliq[0]['CDCSTICMS'];
                          $paramsVendaImpos['CDCSTPRODPC']           = $consultaAliq[0]['CDCSTPISCOF'];
                          $paramsVendaImpos['CDCFOPPROD']            = $consultaAliq[0]['CDCFOPPFIS'];
                           
                           
                            
                          // Executa Insert vendaImpos
                          $resultInsertVendaImpos = $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::QUERY_INSERT_VENDAIMPOS,$paramsVendaImpos,$conn);
                           if($resultInsertVendaImpos->error()){
                                $arrayDados = $this->buildRetorno($arrayDados,'T','E','Erro ao  inserir dados na tabela de imposto','','','','');
                                $this->expImpNfceService->insertMsgLogArquivoXml($arrayDados,$conn);
                                return $arrayDados;
                            }
                          
                        }
                        
                        
                        //Insere MOVCAIXA
                        
                        
                        for ($i = 0; $i <= count($cdsMeioPgtoXml['IDPGTO']) - 1; $i++){
                            
                            // valida recebimento
                                
                                $params['DTABERTURACAIXA'] = $params['DTABERTUR'];
                                $nrSeqMovi = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_GERA_NRSEQMOVI,$params,$conn)->getResult();
                                
                                if($stTipoEmissao == 'N'){
                                    if($cdsMeioPgtoXml['IDPGTO'][$i] == '01'){
                                        $stTipoRece = '4'; //Dinheiro
                                    }else if($cdsMeioPgtoXml['IDPGTO'][$i] == '02'){
                                        $stTipoRece = '3'; //Cheque
                                    }else if($cdsMeioPgtoXml['IDPGTO'][$i] == '03'){
                                        $stTipoRece = '1'; //Cartão de Crédito
                                    }else if($cdsMeioPgtoXml['IDPGTO'][$i] == '04'){
                                        $stTipoRece = '2'; //Cartão de Débito
                                    }else if($cdsMeioPgtoXml['IDPGTO'][$i] == '05'){
                                        $stTipoRece = '9'; //Crédito Pessoal
                                    }else{
                                        $stTipoRece = '4'; //Dinheiro
                                    }
                                }else{
                                    $stTipoRece = '4'; //Dinheiro
                                }
                                
                                
                                $paramsTipoRece['CDFILIAL']   = $stFilial;
                                $paramsTipoRece['CDCAIXA']    = $stCdCaixa;
                                $paramsTipoRece['IDTIPORECE'] = $stTipoRece;
                                
                                $consultaTipoRece = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIFI_TIPORECE,$paramsTipoRece,$conn)->getResult();
                                
                                
                                if(empty($consultaTipoRece)){
                                    $stTipoRece = '001';
                                    $arrayDados = $this->buildRetorno($arrayDados,'F','A','Meio de pagamento "' . $cdsMeioPgtoXml['IDPGTO'][$i] . '" não encontrado na configuração do caixa. Será cadastrado na venda o recebimento de código 001.','Verifique o cadastro na tela Configuração do Terminal de Caixa > selecione a vigência > Grupos de Recebimento.','','',$stTipoRece);
                                    
                                    $this->expImpNfceService->insertMsgLogArquivoXml($arrayDados,$conn);
                                }else{
                                    $stTipoRece = $consultaTipoRece[0]['CDTIPORECE'];
                                }
                                
                                
                               if($cdsMeioPgtoXml['VRPGTO'][$i] == 0){
                                    $arrayDados = $this->buildRetorno($arrayDados,'F','A','Meio de pagamento "' . $cdsMeioPgtoXml['IDPGTO'][$i] . '" possui valor igual a zero no XML. Não será inserido na venda.','Verifique o meio de pagamento no arquivo XML.','','',$cdsMeioPgtoXml['IDPGTO'][$i]);
                                
                                $this->expImpNfceService->insertMsgLogArquivoXml($arrayDados,$conn);
                                }else{
                                    $paramsMovCaixa['CDFILIAL']      = $stFilial;
                                    $paramsMovCaixa['CDCAIXA']       = $stCdCaixa;
                                    $paramsMovCaixa['DTABERCAIX']    = $stDtAberCaixa;    
                                    $paramsMovCaixa['NRSEQUMOVI']    = $nrSeqMovi[0]['NRSEQMOVI'];
                                    
                                    if($cdsMeioPgtoXml['IDPGTO'][$i] != 'Troco'){
                                        $paramsMovCaixa['IDTIPOMOVIVE']  = 'E';  
                                    }else{
                                        $paramsMovCaixa['IDTIPOMOVIVE']  = 'S'; 
                                    }
                                    $paramsMovCaixa['VRMOVIVEND']    = $cdsMeioPgtoXml['VRPGTO'][$i];  
                                    $paramsMovCaixa['NRSEQVENDA']    = $nrSeqVenda[0]['NRSEQVENDA'];
                                    $paramsMovCaixa['CDTIPORECE']    = $stTipoRece;
                                    $paramsMovCaixa['DTHRINCMOV']    = $cdsVendaXml['DTVENDA'];
                                    $paramsMovCaixa['DTMOVIMCAIXA']  = substr(strval($cdsVendaXml['DTVENDA']),0,10);
                                    if($consumidorPadrao){
                                        $paramsMovCaixa['CDCLIENTE'] = $consultaConsumidorPadrao[0]['CDCLIENTE'];
                                    }else{
                                        $paramsMovCaixa['CDCLIENTE'] = $consultaConsumidor[0]['CDCLIENTE'];
                                    }
                                  $resultInsertMovCaixa = $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::QUERY_INSERT_MOVCAIXA,$paramsMovCaixa,$conn);
                                  if($resultInsertMovCaixa->error()){
                                        $arrayDados = $this->buildRetorno($arrayDados,'T','E','Erro ao  inserir dados na tabela de recebimentos','','','','');
                                        $this->expImpNfceService->insertMsgLogArquivoXml($arrayDados,$conn);
                                        return $arrayDados;
                                    }
                                   
                                }   
                            
                        }
                        
                        if(!in_array('T',$arrayDados['error']['ERROSTATUS'])){
                            $arrayDados = $this->buildRetorno($arrayDados,'F','S','Nota importada','','','','');
                            
                            $arrayDados['error']['IDSTATUSIMPARQ']   = 'S';
                            
                            $param['IDSTATUSIMPARQ']  = 'S';
                            $param['NRACESSONFCE']    = $arrayDados['error']['NRACESSONFCE'];
                            $param['NRSEQVENDA']      = $nrSeqVenda[0]['NRSEQVENDA'];
                            
                            $resultUpdateStatusImp  = $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::UPDATE_STATUS_IMP_ARQXML,$param,$conn);
                            
                            $conn->commit();
                            
                            return $arrayDados;
                        }else if(in_array('T',$arrayDados['error']['ERROSTATUS'])){
                            
                            $conn->rollback();
                            return $arrayDados;
                        }
                
                }else{
                    $arrayDados = $this->buildRetorno($arrayDados,'T','E','Venda já existente no sistema.','','','','');
                    
                    $this->expImpNfceService->insertMsgLogArquivoXml($arrayDados,$conn);
                        
                     return $arrayDados;
                }
      
    }
    
    private function setVendaSatCanc ($arrayDados,$conn){
        
        $cdsVendaXml         =  $arrayDados['cdsVendaXml']; 
        $stFilial            =  $arrayDados['stFilial']; 
        $stCdCaixa           =  $arrayDados['stCdCaixa'];
        $stEmpresa           =  $arrayDados['stEmpresa'];           
        $stNomeXml           =  $arrayDados['seFiles'];             
        $stCdOperador        =  $arrayDados['stCdOperador'];
        $arrayDados['error']['NRNOTAFISCALCE']   = $cdsVendaXml['NRNOTAFISCALCE'];
        //Verifica se a venda já existe no banco
        $params['CDFILIAL']       = $stFilial;
        $params['CDCAIXA']        = $stCdCaixa;
        $params['DTENTRVENDA']    = substr($cdsVendaXml['DTVENDA'],0,10);
        $params['CDSERIESAT']     = $cdsVendaXml['CDSERIESAT'];
        $params['NRACESSONFCE']   = $cdsVendaXml['NRACESSONFCE']; 
          
        //busca venda concluida para ser cancelada
        $consultaVendaCanc = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIF_VENDA_SATCANC,$params,$conn)->getResult();
        
        if(!empty($consultaVendaCanc)){
            
            $params['CDFILIAL']       =  $consultaVendaCanc[0]['CDFILIAL'];
            $params['CDCAIXA']        =  $consultaVendaCanc[0]['CDCAIXA'];
            $params['NRSEQVENDA']     =  $consultaVendaCanc[0]['NRSEQVENDA'];
            $params['NRACESSOCANC']   =  $cdsVendaXml['NRACESSOCANC'];
            $params['DSARQXMLCANC']   =  $cdsVendaXml['DSARQXMLNFCE'];
            $resultUpdateVendaCanc     = $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::UPDATE_VENDA_CANC,$params,$conn);
            $resultUpdateMOVCaixaCanc  = $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::UPDATE_MOV_CAIXA_SATCANC,$params,$conn);
            $resultUpdateArqXmlSatCanc = $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::UPDATE_ARQXML_SATCANC,$params,$conn);
            $arrayDados = $this->buildRetorno($arrayDados,'F','S','Nota Importada.','','','','');
            $arrayDados['error']['IDSTATUSIMPARQ']   = 'S';
            
            $params['IDSTATUSIMPARQ']  = 'S';
            $params['NRACESSONFCE']    = $cdsVendaXml['NRACESSOCANC'];
            
            $resultUpdateStatusImp  = $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::UPDATE_STATUS_IMP_ARQXML,$params,$conn);
            $conn->commit();
            
            return $arrayDados;
        }else{
              
            $arrayDados = $this->buildRetorno($arrayDados,'T','E','A venda desta nota não foi encontrada no sistema e não pode ser cancelada.','','','','');
            $arrayDados['error']['IDSTATUSIMPARQ']   = 'N';
                    
            $this->expImpNfceService->insertMsgLogArquivoXml($arrayDados,$conn);
                
                
             return $arrayDados;
        }
        
    }
    
    private function geraXmlSat ($loteCFe,$arrayDados,$stDadosFixos,$connection=null){
         
            $infCFe     = $loteCFe->infCFe;
            $ide        = $infCFe->ide;
            $emit       = $infCFe->emit;
            $enderEmit  = $emit->enderEmit;
            $dest       = $infCFe->dest;
            $total      = $infCFe->total;
            $totIcms    = $total->ICMSTot;
            
            //recebe todo o conteúdo xml da CFe 
            $cdsVendaXml['DSARQXMLNFCE'] = $stDadosFixos['DSARQXMLNFCE'];
            
            //VENDA
            $cdsVendaXml['NRACESSONFCE'] = strval($loteCFe->infCFe->attributes()->Id);  
            $cdsVendaXml['NRINSJURFILI'] = strval($emit->CNPJ);
            $cdsVendaXml['SGESTADO']     = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_GET_SGESTADO,array('CDESTADOIBGE'=>strval($ide->cUF)),$connection)->getResult()[0]['SGESTADO'];
            $cdsVendaXml['CDSERIESAT']   = str_pad($ide->nserieSAT,  3, 0,STR_PAD_LEFT);
            $cdsVendaXml['CDSERIENFCE']  = '';
            
            $stData = $loteCFe->infCFe->ide->dEmi;
            $stHora = $loteCFe->infCFe->ide->hEmi;
            
            $cdsVendaXml['DTVENDA']         = substr($stData,6,2).'/'.substr($stData,4,2).'/'.substr($stData,0,4).' '.substr($stHora,0,2).':'.substr($stHora,2,2).':'.substr($stHora,4,2);
            $cdsVendaXml['NRLANCTONFCE']    = substr($stData,8,2) . $ide->cNF;
            $cdsVendaXml['IDTPAMBNFCE']     = strval($ide->tpAmb);
            $cdsVendaXml['NRNOTAFISCALCE']  = str_pad($ide->nCFe,  6, 0,STR_PAD_LEFT);
            $cdsVendaXml['CDCAIXA']         = strval($ide->numeroCaixa);
            $cdsVendaXml['NRINSCRCONS']     = strval($dest->CPF);
            
            
            //ITEMVENDA-ITVENDAIMPOS
             foreach($infCFe->det as $item){
                 $prod = $item->prod;
                 $cdsItensXml['CDPRODUTO'][]     = strval($prod->cProd);
                 $cdsItensXml['NMPRODUTO'][]     = strval($prod->xProd);
                 $cdsItensXml['QTPRODVEND'][]    = floatval($prod->qCom);
                 $cdsItensXml['VRUNITVEND'][]    = floatval($prod->vUnCom);
                 $cdsItensXml['NRSEQUITVEND'][]  = str_pad($item->attributes()->nItem,  3, 0,STR_PAD_LEFT);
                 
                 foreach($item->imposto as $imposto){
                    $ICMS     = $imposto->ICMS;
                    $PIS     = $imposto->PIS;
                    $COFINS     = $imposto->COFINS;
                    
                    if (isset($PIS->PISAliq->pPIS)) {
                        $cdsItensXml['PRPIS'][]    = floatval($PIS->PISAliq->pPIS);
                    }
                    else{
                       $cdsItensXml['PRPIS'][] = 0;
                    }
                    if (isset($PIS->PISAliq->vPIS)) {
                        $cdsItensXml['VPIS'][]    = floatval($PIS->PISAliq->vPIS);
                    }
                    else{
                       $cdsItensXml['VPIS'][] = 0;
                    }
                    if (isset($COFINS->COFINSAliq->pCOFINS)) {
                        $cdsItensXml['PRCOFIN'][]    = floatval($COFINS->COFINSAliq->pCOFINS);
                    }
                    else{
                       $cdsItensXml['PRCOFIN'][] = 0;
                    }
                    if (isset($COFINS->COFINSAliq->vCOFINS)) {
                        $cdsItensXml['VCOFIN'][]    = floatval($COFINS->COFINSAliq->vCOFINS);
                    }
                    else{
                       $cdsItensXml['VCOFIN'][] = 0;
                    }
                    
                    if(isset($ICMS->ICMS00->pICMS) && isset($ICMS->ICMS00->vICMS)) {
                        $cdsItensXml['PRICMS'][] = floatval($ICMS->ICMS00->pICMS);
                        $cdsItensXml['VICMS'][]  = floatval($ICMS->ICMS00->vICMS);
                    } else if(isset($ICMS->ICMS10->pICMS) && isset($ICMS->ICMS10->vICMS)) {
                        $cdsItensXml['PRICMS'][] = floatval($ICMS->ICMS10->pICMS);
                        $cdsItensXml['VICMS'][]  = floatval($ICMS->ICMS10->vICMS);
                    } else if(isset($ICMS->ICMS20->pICMS) && isset($ICMS->ICMS20->vICMS)) {
                        $cdsItensXml['PRICMS'][] = floatval($ICMS->ICMS20->pICMS);
                        $cdsItensXml['VICMS'][]  = floatval($ICMS->ICMS20->vICMS);
                    } else if(isset($ICMS->ICMS30->pICMS) && isset($ICMS->ICMS30->vICMS)) {
                        $cdsItensXml['PRICMS'][] = floatval($ICMS->ICMS30->pICMS);
                        $cdsItensXml['VICMS'][]  = floatval($ICMS->ICMS30->vICMS);
                    } else if(isset($ICMS->ICMS40->pICMS) && isset($ICMS->ICMS40->vICMS)) {
                        $cdsItensXml['PRICMS'][] = floatval($ICMS->ICMS40->pICMS);
                        $cdsItensXml['VICMS'][]  = floatval($ICMS->ICMS40->vICMS);
                    } else if(isset($ICMS->ICMS41->pICMS) && isset($ICMS->ICMS41->vICMS)) {
                        $cdsItensXml['PRICMS'][] = floatval($ICMS->ICMS41->pICMS);
                        $cdsItensXml['VICMS'][]  = floatval($ICMS->ICMS41->vICMS);
                    } else if(isset($ICMS->ICMS50->pICMS) && isset($ICMS->ICMS50->vICMS)) {
                        $cdsItensXml['PRICMS'][] = floatval($ICMS->ICMS50->pICMS);
                        $cdsItensXml['VICMS'][]  = floatval($ICMS->ICMS50->vICMS);
                    } else if(isset($ICMS->ICMS51->pICMS) && isset($ICMS->ICMS51->vICMS)) {
                        $cdsItensXml['PRICMS'][] = floatval($ICMS->ICMS51->pICMS);
                        $cdsItensXml['VICMS'][]  = floatval($ICMS->ICMS51->vICMS);
                    } else if(isset($ICMS->ICMS60->pICMS) && isset($ICMS->ICMS60->vICMS)) {
                        $cdsItensXml['PRICMS'][] = floatval($ICMS->ICMS60->pICMS);
                        $cdsItensXml['VICMS'][]  = floatval($ICMS->ICMS60->vICMS);
                    } else if(isset($ICMS->ICMS70->pICMS) && isset($ICMS->ICMS70->vICMS)) {
                        $cdsItensXml['PRICMS'][] = floatval($ICMS->ICMS70->pICMS);
                        $cdsItensXml['VICMS'][]  = floatval($ICMS->ICMS70->vICMS);
                    } else if(isset($ICMS->ICMS90->pICMS) && isset($ICMS->ICMS90->vICMS)) {
                        $cdsItensXml['PRICMS'][] = floatval($ICMS->ICMS90->pICMS);
                        $cdsItensXml['VICMS'][]  = floatval($ICMS->ICMS90->vICMS);
                    } else {
                        $cdsItensXml['PRICMS'][] = 0;
                        $cdsItensXml['VICMS'][]  = 0;
                    }
                 }
                 //Imposto SAT
                 
                 // verifica se existe rateio de desconto e acrescimo no item
                 if(!is_null($prod->vRatDesc)){
                     $vRatDesc = floatval($prod->vRatDesc);
                 }else{
                     $vRatDesc = 0;
                 }
                 
                 if(!is_null($prod->vRatAcr)){
                     $vRatAcr = floatval($prod->vRatAcr);
                 }else{
                     $vRatAcr = 0;
                 }
                 
                 
                 // verifica se existe acrescimo e desconto e soma com o respectivo rateio ()
                 if(!is_null($prod->vDesc)){
                     $cdsItensXml['VRDESITVEND'][] = floatval($prod->vDesc) + $vRatDesc;
                 }else {
                     $cdsItensXml['VRDESITVEND'][] = 0 + $vRatDesc;
                 }
                 
                 if(!is_null($prod->vOutro)){
                     $cdsItensXml['VRACRITVEND'][] = floatval($prod->vOutro) + $vRatAcr;
                 }else{
                     $cdsItensXml['VRACRITVEND'][] = 0 + $vRatAcr;
                 }
                 
                
             }
             
             //RECEBIMENTOS
              foreach($infCFe->pgto->MP as $pag){
                  if($pag){
                      $cdsMeioPgtoXml['IDPGTO'][] = strval($pag->cMP);
                      $cdsMeioPgtoXml['VRPGTO'][] = floatval($pag->vMP);
                  }
              }
              if($infCFe->pgto->vTroco){
                      $cdsMeioPgtoXml['IDPGTO'][] = 'Troco';
                      $cdsMeioPgtoXml['VRPGTO'][] = floatval($infCFe->pgto->vTroco);
                  }
              
              // valida se existe filial no sistema usando o CNPJ coletado do  arquivo
                  
                  $params['NRINSJURFILI'] = $cdsVendaXml['NRINSJURFILI'];
                  
                //   $params['CDSERIECX'] = $cdsVendaXml['CDSERIESAT'];
                  $consultaFilialSat  = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_GET_FILIAL_SAT,$params,$connection)->getResult();
                 
                // se nao existir filial, utiliza os dados de unidade informados
              if(!empty($consultaFilialSat)){
                  $stEmpresa    = $consultaFilialSat[0]['CDEMPRESA'];
                  $stCdCaixa    = $consultaFilialSat[0]['CDCAIXA'];
                  $stFilial     = $consultaFilialSat[0]['CDFILIAL'];
                  
                }else if(!empty($stDadosFixos['CDFILIAL'])){
                    $consultaFilial  = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_GET_FILIAL,array('CDFILIAL'=>$stDadosFixos['CDFILIAL']),$connection)->getResult();
                    if($consultaFilial[0]['CDFILIAL'] != $stDadosFixos['CDFILIAL']){
                        $arrayDados = $this->buildRetorno($arrayDados,'T','E','Filial informada nao existe no sistema','Informe a filial correta para importação das vendas na tela Importação do Arquivo XML (SAT).','','','');
                        
                        return $arrayDados;
                    }else{
                        $stEmpresa    = $stDadosFixos['CDEMPRESA'];
                        $stCdCaixa    = $stDadosFixos['CDCAIXA'];
                        $stFilial     = $stDadosFixos['CDFILIAL'];
                        
                    }
                }else{
                    $arrayDados = $this->buildRetorno($arrayDados,'T','E','Filial e caixa não encontrados para o CNPJ do arquivo XML','Informe a filial correta para importação das vendas na tela Importação do Arquivo XML (SAT).','','','');
                    
                    return $arrayDados;
                }
                
                
                
              // parametros com dados do arquivo 
              $arrayDados['isSat']               = true;          
              $arrayDados['cdsVendaXml']         = $cdsVendaXml;
              $arrayDados['cdsItensXml']         = $cdsItensXml;
              $arrayDados['cdsMeioPgtoXml']      = $cdsMeioPgtoXml;
              $arrayDados['stFilial']            = $stFilial;
              $arrayDados['stCdCaixa']           = $stCdCaixa;
              $arrayDados['stEmpresa']           = $stEmpresa;
              $arrayDados['stTipoEmissao']       = 'S';
              $arrayDados['seFiles']             = $stDadosFixos['NMARQUIVOXML'];
              $arrayDados['stCdOperador']        = $stDadosFixos['CDOPERADOR'];
            
            return $arrayDados;
     }
     
    private function geraXmlSatCanc ($loteCFe,$arrayDados,$stDadosFixos,$connection=null){
         
            $infCFe     = $loteCFe->infCFe;
            $ide        = $infCFe->ide;
            $emit       = $infCFe->emit;
            
            $stData     = $loteCFe->infCFe->ide->dEmi;
            
            $cdsVendaXml['DSARQXMLNFCE']         = $stDadosFixos['DSARQXMLNFCE'];
            $cdsVendaXml['NRACESSONFCE']         = strval($loteCFe->infCFe->attributes()->chCanc);
            $cdsVendaXml['NRACESSOCANC']         = strval($loteCFe->infCFe->attributes()->Id);
            $cdsVendaXml['DTVENDA']              = substr($stData,6,2).'/'.substr($stData,4,2).'/'.substr($stData,0,4);
            $cdsVendaXml['NRINSJURFILI']         = strval($emit->CNPJ);
            $cdsVendaXml['SGESTADO']             = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_GET_SGESTADO,array('CDESTADOIBGE'=>strval($ide->cUF)),$connection)->getResult()[0]['SGESTADO'];
            $cdsVendaXml['NRNOTAFISCALCE']       = str_pad($ide->nCFe,  6, 0,STR_PAD_LEFT);
            $cdsVendaXml['CDSERIESAT']           = str_pad($ide->nserieSAT,  3, 0,STR_PAD_LEFT);
            $cdsVendaXml['CDSERIENFCE']          = '';
            $cdsVendaXml['CDCAIXA']              = strval($ide->numeroCaixa);
            
            
            // valida se existe filial no sistema usando o CNPJ coletado do  arquivo
                  
                  $params['NRINSJURFILI'] = $cdsVendaXml['NRINSJURFILI'];
                //   $params['CDSERIECX'] = $cdsVendaXml['CDSERIESAT'];
                  $consultaFilialSat  = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_GET_FILIAL_SAT,$params,$connection)->getResult();
                 
                // se nao existir filial, utiliza os dados de unidade informados
              if(!empty($consultaFilialSat)){
                  $stEmpresa    = $consultaFilialSat[0]['CDEMPRESA'];
                  $stCdCaixa    = $consultaFilialSat[0]['CDCAIXA'];
                  $stFilial     = $consultaFilialSat[0]['CDFILIAL'];
                  
                }else if(!empty($stDadosFixos['CDFILIAL'])){
                    $consultaFilial  = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_GET_FILIAL,array('CDFILIAL'=>$stDadosFixos['CDFILIAL']),$connection)->getResult();
                    if($consultaFilial[0]['CDFILIAL'] != $stDadosFixos['CDFILIAL']){
                        $arrayDados = $this->buildRetorno($arrayDados,'T','E','Filial informada nao existe no sistema','Informe a filial correta para importação das vendas na tela Importação do Arquivo XML (SAT).','','','');
                        
                        return $arrayDados;
                    }else{
                        $stEmpresa    = $stDadosFixos['CDEMPRESA'];
                        $stCdCaixa    = $stDadosFixos['CDCAIXA'];
                        $stFilial     = $stDadosFixos['CDFILIAL'];
                        
                    }
                }else{
                    $arrayDados = $this->buildRetorno($arrayDados,'T','E','Filial e caixa não encontrados para o CNPJ do arquivo XML','Informe a filial correta para importação das vendas na tela Importação do Arquivo XML (SAT).','','','');
                    
                    return $arrayDados;
                }
            
            // parametros com dados do arquivo
            $arrayDados['isSat']               = true;          
            $arrayDados['cdsVendaXml']         = $cdsVendaXml;
            $arrayDados['stFilial']            = $stFilial;
            $arrayDados['stCdCaixa']           = $stCdCaixa;
            $arrayDados['stEmpresa']           = $stEmpresa;
            $arrayDados['seFiles']             = $stDadosFixos['NMARQUIVOXML'];
            $arrayDados['stCdOperador']        = $stDadosFixos['CDOPERADOR'];
            
            return $arrayDados;
     }
    
    private function geraXmlNfce ($loteNFCe,$protNFe,$arrayDados,$stDadosFixos,$connection=null){
        
            $infCFe     = $loteNFCe->infNFe;
            $ide        = $infCFe->ide;
            $emit       = $infCFe->emit;
            $enderEmit  = $emit->enderEmit;
            $dest       = $infCFe->dest;
            $total      = $infCFe->total;
            $totIcms    = $total->ICMSTot;
            
            
            //recebe todo o conteúdo xml do arquivo
            $cdsVendaXml['DSARQXMLNFCE'] = $stDadosFixos['DSARQXMLNFCE'];
            
            //VENDA
            $cdsVendaXml['NRACESSONFCE'] = substr($loteNFCe->infNFe->attributes()->Id, 3, strlen($loteNFCe->infNFe->attributes()->Id));  
            $cdsVendaXml['NRINSJURFILI'] = strval($emit->CNPJ);
            $cdsVendaXml['SGESTADO']     = strval($enderEmit->UF);
            $cdsVendaXml['CDSERIENFCE']  = str_pad($ide->serie,  3, 0,STR_PAD_LEFT); 
            $cdsVendaXml['CDSERIESAT']   = '';
            
            
            
            $stData = $ide->dhEmi;
    
            $cdsVendaXml['DTVENDA']         = date('d/m/Y h:i:s', strtotime($stData));
            $cdsVendaXml['NRLANCTONFCE']    = substr($stData,8,2) . $ide->cNF;
            $cdsVendaXml['IDTPAMBNFCE']     = strval($ide->tpAmb);
            $cdsVendaXml['NRNOTAFISCALCE']  = str_pad($ide->nNF,  6, 0,STR_PAD_LEFT);
            $cdsVendaXml['NRINSCRCONS']     = strval($dest->CPF);
            
          
            
            $infProt = $protNFe->infProt;
            $stData  = $infProt->dhRecbto;
            $cdsVendaXml['DTHRPROTOCONFCE'] = date('d/m/Y h:i:s', strtotime($stData));
            $cdsVendaXml['NRPROTOCOLONFCE'] = strval($infProt->nProt);
            $cdsVendaXml['DSOBSSTATUSNFCE'] = strval($infProt->cStat . ' - ' . $infProt->xMotivo);
            
            
            
                    
             //ITEMVENDA-ITVENDAIMPOS
             foreach($infCFe->det as $item){
                 $prod = $item->prod;
                 $cdsItensXml['CDPRODUTO'][]     = strval($prod->cProd);
                 $cdsItensXml['NMPRODUTO'][]     = strval($prod->xProd);
                 $cdsItensXml['QTPRODVEND'][]    = floatval($prod->qCom);
                 $cdsItensXml['VRUNITVEND'][]    = floatval($prod->vUnCom);
                 $cdsItensXml['NRSEQUITVEND'][]  = str_pad($item->attributes()->nItem,  3, 0,STR_PAD_LEFT);
                 
                 foreach($item->imposto as $imposto){
                    $ICMS     = $imposto->ICMS;
                    $PIS     = $imposto->PIS;
                    $COFINS     = $imposto->COFINS;
                    
                    if (isset($PIS->PISAliq->pPIS)) {
                        $cdsItensXml['PRPIS'][]    = floatval($PIS->PISAliq->pPIS);
                    }
                    else{
                        $cdsItensXml['PRPIS'][] = 0;
                    }
                    if (isset($PIS->PISAliq->vPIS)) {
                        $cdsItensXml['VPIS'][]    = floatval($PIS->PISAliq->vPIS);
                    }
                    else{
                        $cdsItensXml['VPIS'][] = 0;
                    }
                    if (isset($COFINS->COFINSAliq->pCOFINS)) {
                        $cdsItensXml['PRCOFIN'][]    = floatval($COFINS->COFINSAliq->pCOFINS);
                    }
                    else{
                        $cdsItensXml['PRCOFIN'][] = 0;
                    }
                    if (isset($COFINS->COFINSAliq->vCOFINS)) {
                        $cdsItensXml['VCOFIN'][]    = floatval($COFINS->COFINSAliq->vCOFINS);
                    }
                    else{
                        $cdsItensXml['VCOFIN'][] = 0;
                    }
                    
                    if(isset($ICMS->ICMS00->pICMS) && isset($ICMS->ICMS00->vICMS)) {
                        $cdsItensXml['PRICMS'][] = floatval($ICMS->ICMS00->pICMS);
                        $cdsItensXml['VICMS'][]  = floatval($ICMS->ICMS00->vICMS);
                    } else if(isset($ICMS->ICMS10->pICMS) && isset($ICMS->ICMS10->vICMS)) {
                        $cdsItensXml['PRICMS'][] = floatval($ICMS->ICMS10->pICMS);
                        $cdsItensXml['VICMS'][]  = floatval($ICMS->ICMS10->vICMS);
                    } else if(isset($ICMS->ICMS20->pICMS) && isset($ICMS->ICMS20->vICMS)) {
                        $cdsItensXml['PRICMS'][] = floatval($ICMS->ICMS20->pICMS);
                        $cdsItensXml['VICMS'][]  = floatval($ICMS->ICMS20->vICMS);
                    } else if(isset($ICMS->ICMS30->pICMS) && isset($ICMS->ICMS30->vICMS)) {
                        $cdsItensXml['PRICMS'][] = floatval($ICMS->ICMS30->pICMS);
                        $cdsItensXml['VICMS'][]  = floatval($ICMS->ICMS30->vICMS);
                    } else if(isset($ICMS->ICMS40->pICMS) && isset($ICMS->ICMS40->vICMS)) {
                        $cdsItensXml['PRICMS'][] = floatval($ICMS->ICMS40->pICMS);
                        $cdsItensXml['VICMS'][]  = floatval($ICMS->ICMS40->vICMS);
                    } else if(isset($ICMS->ICMS41->pICMS) && isset($ICMS->ICMS41->vICMS)) {
                        $cdsItensXml['PRICMS'][] = floatval($ICMS->ICMS41->pICMS);
                        $cdsItensXml['VICMS'][]  = floatval($ICMS->ICMS41->vICMS);
                    } else if(isset($ICMS->ICMS50->pICMS) && isset($ICMS->ICMS50->vICMS)) {
                        $cdsItensXml['PRICMS'][] = floatval($ICMS->ICMS50->pICMS);
                        $cdsItensXml['VICMS'][]  = floatval($ICMS->ICMS50->vICMS);
                    } else if(isset($ICMS->ICMS51->pICMS) && isset($ICMS->ICMS51->vICMS)) {
                        $cdsItensXml['PRICMS'][] = floatval($ICMS->ICMS51->pICMS);
                        $cdsItensXml['VICMS'][]  = floatval($ICMS->ICMS51->vICMS);
                    } else if(isset($ICMS->ICMS60->pICMS) && isset($ICMS->ICMS60->vICMS)) {
                        $cdsItensXml['PRICMS'][] = floatval($ICMS->ICMS60->pICMS);
                        $cdsItensXml['VICMS'][]  = floatval($ICMS->ICMS60->vICMS);
                    } else if(isset($ICMS->ICMS70->pICMS) && isset($ICMS->ICMS70->vICMS)) {
                        $cdsItensXml['PRICMS'][] = floatval($ICMS->ICMS70->pICMS);
                        $cdsItensXml['VICMS'][]  = floatval($ICMS->ICMS70->vICMS);
                    } else if(isset($ICMS->ICMS90->pICMS) && isset($ICMS->ICMS90->vICMS)) {
                        $cdsItensXml['PRICMS'][] = floatval($ICMS->ICMS90->pICMS);
                        $cdsItensXml['VICMS'][]  = floatval($ICMS->ICMS90->vICMS);
                    } else {
                        $cdsItensXml['PRICMS'][] = 0;
                        $cdsItensXml['VICMS'][]  = 0;
                    }
                 }
                 
                 if($prod->vDesc){
                     $cdsItensXml['VRDESITVEND'][] = floatval($prod->vDesc);
                 }else {
                     $cdsItensXml['VRDESITVEND'][] = 0;
                 }
                 
                 if($prod->vOutro){
                     $cdsItensXml['VRACRITVEND'][] = floatval($prod->vOutro);
                 }else{
                     $cdsItensXml['VRACRITVEND'][] = 0;
                 }
             }
             $cdsMeioPgtoXml = [];
             //RECEBIMENTOS
              foreach($infCFe->pag->detPag as $pag){
                  if(isset($pag->tPag)){
                        $cdsMeioPgtoXml['IDPGTO'][] = strval($pag->tPag);
                        $cdsMeioPgtoXml['VRPGTO'][] = floatval($pag->vPag);
                  }
              }
              if($infCFe->pag->vTroco){
                  $cdsMeioPgtoXml['IDPGTO'][] = 'Troco';
                  $cdsMeioPgtoXml['VRPGTO'][] = floatval($infCFe->pag->vTroco);
              }
              
             
            // valida se existe filial no sistema usando o CNPJ coletado do  arquivo
                  
                  $params['NRINSJURFILI'] = $cdsVendaXml['NRINSJURFILI'];
                  $consultaFilialNfce  = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_GET_FILIAL_NFCE,$params,$connection)->getResult();
                 
                // se nao existir filial, utiliza os dados de unidade informados
              if(!empty($consultaFilialNfce)){
                  $stEmpresa    = $consultaFilialNfce[0]['CDEMPRESA'];
                  $stCdCaixa    = $consultaFilialNfce[0]['CDCAIXA'];
                  $stFilial     = $consultaFilialNfce[0]['CDFILIAL'];
                  
                }else if(!empty($stDadosFixos['CDFILIAL'])){
                    $consultaFilial  = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_GET_FILIAL,array('CDFILIAL'=>$stDadosFixos['CDFILIAL']),$connection)->getResult();
                    if($consultaFilial[0]['CDFILIAL'] != $stDadosFixos['CDFILIAL']){
                        $arrayDados = $this->buildRetorno($arrayDados,'T','E','Filial informada não existe no sistema','Informe a filial correta para importação das vendas na tela Importação do Arquivo XML (NFC-e).','','','');
                        
                        return $arrayDados;
                    }else{
                        $stEmpresa    = $stDadosFixos['CDEMPRESA'];
                        $stCdCaixa    = $stDadosFixos['CDCAIXA'];
                        $stFilial     = $stDadosFixos['CDFILIAL'];
                    }
                }else{
                    $arrayDados = $this->buildRetorno($arrayDados,'T','E','Filial e caixa não encontrados para o CNPJ e a série do arquivo XML','Informe a filial correta para importação das vendas na tela Importação do Arquivo XML (NFC-e).','','','');
                    
                    return $arrayDados;
                }
                
              // parametros com dados do arquivo 
              $arrayDados['isSat']               = false;
              $arrayDados['cdsVendaXml']         = $cdsVendaXml;
              $arrayDados['cdsItensXml']         = $cdsItensXml;
              $arrayDados['cdsMeioPgtoXml']      = $cdsMeioPgtoXml;
              $arrayDados['stFilial']            = $stFilial;
              $arrayDados['stCdCaixa']           = $stCdCaixa;
              $arrayDados['stEmpresa']           = $stEmpresa;
              $arrayDados['stTipoEmissao']       = 'N';
              $arrayDados['seFiles']             = $stDadosFixos['NMARQUIVOXML'];
              $arrayDados['stCdOperador']        = $stDadosFixos['CDOPERADOR'];
             
              
              
              return $arrayDados;
       
    }
    
    private function remove_utf8_bom($text){
        $bom = pack('H*','EFBBBF');
        $text = preg_replace("/^$bom/", '', $text);
        return $text;
    }
    
    private function verificaArqSatNfce($arquivo){
        
        $arquivo = $this->remove_utf8_bom($arquivo);
        if(!mb_check_encoding($arquivo, 'UTF-8')){
            $arquivo = utf8_encode($arquivo);
        }
        $arquivo = mb_substr($arquivo,strpos($arquivo,'<'));
        
        if(strpos($arquivo,'<LoteCFeCanc>') !== false){
            $xmlFileObject = simplexml_load_string($arquivo,null,LIBXML_COMPACT);
            $xmlFileObject->LoteCFeCanc ? $xmlFile = $xmlFileObject->LoteCFeCanc : null;
            
            return  $xmlFile;   
        }else if(strpos($arquivo,'<LoteCFe>') !== false){   
            $xmlFileObject = simplexml_load_string($arquivo,null,LIBXML_COMPACT);
            $xmlFileObject->LoteCFe ? $xmlFile = $xmlFileObject->LoteCFe : null;
            
            return  $xmlFile; 
        }else if(strpos($arquivo,'<CFe>') !== false){
            //adiciona o node 'Sat' para garantir a leitura do node 'Cfe' corretamente
            $nodePosIni = strpos($arquivo,'<CFe>');
            $nodePosEnd = strpos($arquivo,'</CFe>');
            
            $arquivo = substr_replace($arquivo,'<Sat>',$nodePosIni,0);
            $arquivo = substr_replace($arquivo,'</Sat>',$nodePosEnd + 11,0);
            
            $xmlFile = simplexml_load_string($arquivo,null,LIBXML_COMPACT);
            
            return  $xmlFile; 
        }else if( strpos($arquivo,'<CFeCanc>') !== false){
             //arquivo de venda cancelada SAT
            //adiciona o node 'Sat' para garantir a leitura do node 'CFeCanc' corretamente
            $nodePosIni = strpos($arquivo,'<CFeCanc>');
            $nodePosEnd = strpos($arquivo,'</CFeCanc>');
            
            $arquivo = substr_replace($arquivo,'<Sat>',$nodePosIni,0);
            $arquivo = substr_replace($arquivo,'</Sat>',$nodePosEnd + 15,0);
            
            $xmlFile = simplexml_load_string($arquivo,null,LIBXML_COMPACT);
            
            return  $xmlFile; 
        }else if(strpos($arquivo,'<NFe') !== false){
            
            if (strrpos($arquivo,'mod>65</mod>') == false){
                throw new \Exception($e);
            }
            if (strrpos($arquivo,'<protNFe') == false){
                throw new \Exception($e);
            }
            
            $xmlFile = simplexml_load_string($arquivo,null,LIBXML_COMPACT);
            
            return  $xmlFile; 
        }
        else if(strpos($arquivo,'<ProcInutNFe') !== false){


            if (strrpos($arquivo,'mod>65</mod>') == false){
                throw new \Exception($e);
            }

            $xmlFile = simplexml_load_string($arquivo,null,LIBXML_COMPACT);
            return  $xmlFile; 
        }
        
        else if(strpos($arquivo,'<procEventoNFe') !== false && 
             strpos($arquivo,'<evento') !== false &&
             strpos($arquivo,'<retEvento') !== false){
                 
            $xmlFile = simplexml_load_string($arquivo,null,LIBXML_COMPACT);
            return $xmlFile;
        }
        else{
            throw new Exception($e);
        }
    }
    
    private function gravaArqXml($arquivo,$stDadosFixos,$conn=null){
        try{    
            if($conn){
                $connection = $conn;
                
            }else{

                $connection = $this->entityManager->getConnection();
            }
                   
            try{
                // Verifica se o arquivo é do tipo SAT e proveniente do sistema SEFAZ ou de outros buscando os nodes identificados de cada tipo e faz a leitura da string do arquivo XML
                $xmlFile = $this->verificaArqSatNfce($arquivo);
            }catch(\Exception $e){
                $arrayDados['error']['NMARQUIVOXML']     = $stDadosFixos['NMARQUIVOXML'];
                $arrayDados['error']['IDSTATUSIMPARQ']   = 'N';
                $arrayDados = $this->buildRetorno($arrayDados,'T','E','Arquivo XML corrompido ou não é do tipo SAT/NFCe','','','','');
                return $arrayDados['error'];
            }
            
            // Valida o operador
            if(is_null($stDadosFixos['CDOPERADOR'])){
                $params['NRORG']  = $stDadosFixos['NRORG'];
                $consultaOperador = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_GET_OPERADOR,$params,$connection)->getResult();
                $stDadosFixos['CDOPERADOR']  = $consultaOperador[0]['CDOPERADOR'];
            }
            
            
            // Verifica se o arquivo é do tipo NFC-e
            if(count($xmlFile->NFe) && ((string)$xmlFile->NFe->infNFe->ide->mod == '65')){
                
                $connection->beginTransaction();
                
                $arrayDados['IDTIPOARQ']               = 'N';
                $arrayDados['error']['ERROSTATUS'][]   = 'F';
                $arrayDados['stNrOrg']                 = $stDadosFixos['NRORG']; 
                $arrayDados['error']['NMARQUIVOXML']   = $stDadosFixos['NMARQUIVOXML'];
                $arrayDados['error']['NRACESSONFCE']   = substr($xmlFile->NFe->infNFe->attributes()->Id, 3, strlen($xmlFile->NFe->infNFe->attributes()->Id));
                $arrayDados['error']['NRNOTAFISCALCE'] = str_pad($xmlFile->NFe->infNFe->ide->nNF,  6, 0,STR_PAD_LEFT);
                
                $params['NRACESSONFCE']       = substr($xmlFile->NFe->infNFe->attributes()->Id, 3, strlen($xmlFile->NFe->infNFe->attributes()->Id));
                $params['DSNOMEARQUIVO']      = $stDadosFixos['NMARQUIVOXML'];           
                $params['DSARQXMLNFCE']       = $xmlFile->asXML(); 
                $params['DTENTRVENDA']        = date('d/m/Y', strtotime($xmlFile->NFe->infNFe->ide->dhEmi));     
                $params['CDFILIAL']           = $stDadosFixos['CDFILIAL'];        
                $params['CDCAIXA']            = $stDadosFixos['CDCAIXA'];  
                $params['NRORG']              = $stDadosFixos['NRORG'];
                $params['NRIMPARQXMLNFCE']    = NULL;
                $params['IDSTATUSIMPARQ']     = 'N';
                $params['IDTIPOARQ']          = $arrayDados['IDTIPOARQ'];
                $params['CDOPERADOR']         = $stDadosFixos['CDOPERADOR'];
                
                
               //Insere os dados do arquivo no banco
                try{
                    $arrayDados['error']['IDSTATUSIMPARQ']  = 'N';
                    $result = $this->expImpNfceService->insertLogArquivoXml($params,$connection);
                    
                    $arrayDados['error']['NRIMPARQXMLNFCE'] = $result['P_NRIMPARQXMLNFCE'];
                    $arrayDados['error']['IDSTATUSIMPARQ']  = $result['P_IDSTATUSIMPARQ'];
                   
                    if($result['P_IDSTATUSIMPARQ'] == 'S'){
                        $arrayDados = $this->buildRetorno($arrayDados,'T','E','Nota já importada','','','','');
                        $connection->rollback();
                        return $arrayDados['error'];
                    }
                }
                catch(\Exception $e){
                    $msg = $e->getMessage();
                    if(strpos($msg, 'ERRO_MESMO_INSTANTE') !== false){
                        $arrayDados = $this->buildRetorno($arrayDados,'T','E','Arquivo em processamento no momento. Tente novamente mais tarde.','','','','');
                        $connection->rollback();
                        return $arrayDados['error'];
                    }
                    else{
                        throw $e;
                    }
                }
                
                $loteNFCe = $xmlFile->NFe;
                $protNFe = $xmlFile->protNFe;
                
                $arrayDados = $this->geraXmlNfce($loteNFCe,$protNFe,$arrayDados,$stDadosFixos,$connection);
                
                if(in_array('T',$arrayDados['error']['ERROSTATUS'])){
                    $connection->rollback();
                    return $arrayDados['error'];
                }
            
                $arrayDados = $this->setVendas($arrayDados,$connection);
                
                if(in_array('T',$arrayDados['error']['ERROSTATUS'])){
                    $connection->rollback();
                }
               
                return $arrayDados['error'];
                
            // Verifica se o arquivo é do tipo SAT
            }else if(count($xmlFile->CFe)){
                
               
                $arrayDados = array();
               
                foreach($xmlFile->CFe as $CFe){
                    
                    $connection->beginTransaction();
                    
                    $stDadosFixos['DSARQXMLNFCE']             = $CFe->asXML();
                    $arrayDadosAux['IDTIPOARQ']               = 'S';
                    $arrayDadosAux['error']['ERROSTATUS'][]   = 'F';
                    $arrayDadosAux['stNrOrg']                 = $stDadosFixos['NRORG']; 
                    $arrayDadosAux['error']['NMARQUIVOXML']   = $stDadosFixos['NMARQUIVOXML'];
                    $arrayDadosAux['error']['NRACESSONFCE']   = strval($CFe->infCFe->attributes()->Id); 
                    $arrayDadosAux['error']['NRNOTAFISCALCE'][] = str_pad($CFe->infCFe->ide->nCFe,  6, 0,STR_PAD_LEFT);
                    
                    $stData         = $CFe->infCFe->ide->dEmi;
                    $stHora         = $CFe->infCFe->ide->hEmi;
                    
                    $stDataVenda    = substr($stData,6,2).'/'.substr($stData,4,2).'/'.substr($stData,0,4);
                    $params['NRACESSONFCE']       = strval($CFe->infCFe->attributes()->Id);
                    $params['DSNOMEARQUIVO']      = $stDadosFixos['NMARQUIVOXML'];           
                    $params['DSARQXMLNFCE']       = $stDadosFixos['DSARQXMLNFCE'];
                    $params['DTENTRVENDA']        = $stDataVenda;
                    $params['CDFILIAL']           = $stDadosFixos['CDFILIAL'];        
                    $params['CDCAIXA']            = $stDadosFixos['CDCAIXA'];  
                    $params['NRORG']              = $stDadosFixos['NRORG'];
                    $params['CDOPERADOR']         = $stDadosFixos['CDOPERADOR'];
                    $params['NRIMPARQXMLNFCE']    = NULL;
                    $params['IDSTATUSIMPARQ']     = 'N';
                    $params['IDTIPOARQ']          = $arrayDadosAux['IDTIPOARQ'];
                   //Insere os dados do arquivo no banco
                   
                    try{
                        $arrayDadosAux['error']['IDSTATUSIMPARQ']  = 'N';
                        $result = $this->expImpNfceService->insertLogArquivoXml($params,$connection);
                        
                        $arrayDadosAux['error']['NRIMPARQXMLNFCE'] = $result['P_NRIMPARQXMLNFCE'];
                        $arrayDadosAux['error']['IDSTATUSIMPARQ']  = $result['P_IDSTATUSIMPARQ'];
                        
                        if($result['P_IDSTATUSIMPARQ'] == 'S'){
                            $arrayDadosAux = $this->buildRetorno($arrayDadosAux,'T','A','Nota já importada','','','','');
                            $arrayDados[] = $arrayDadosAux['error'];
                            unset($arrayDadosAux);
                            $connection->commit();
                            continue;
                        }
                    }
                    catch(\Exception $e){
                        $msg = $e->getMessage();
                        if(strpos($msg, 'ERRO_MESMO_INSTANTE') !== false){
                            $arrayDadosAux = $this->buildRetorno($arrayDadosAux,'T','E','Arquivo em processamento no momento. Tente novamente mais tarde.','','','','');
                            $arrayDados[] = $arrayDadosAux['error'];
                            unset($arrayDadosAux);
                            $connection->rollback();
                            continue;
                        }
                        else{
                            throw $e;
                        }
                    }
                    
                    $arrayDadosAux = $this->geraXmlSat($CFe,$arrayDadosAux,$stDadosFixos,$connection);
                    if(in_array('T',$arrayDadosAux['error']['ERROSTATUS'])){
                        $connection->rollback();
                        continue;
                    }
                    
                    $arrayDadosAux = $this->setVendas($arrayDadosAux,$connection);
                    if(in_array('T',$arrayDadosAux['error']['ERROSTATUS'])){
                        $connection->rollback();
                    }
                    $arrayDados[] = $arrayDadosAux['error'];
                    
                    unset($arrayDadosAux);
                }
                
                return $arrayDados;
                
                
            // Verifica se o arquivo é do tipo SAT cancelado    
            }else if(count($xmlFile->CFeCanc)){
            
               
                $arrayDados = array();
               
                foreach($xmlFile->CFeCanc as $CFeCanc){
                    
                    $connection->beginTransaction();  
                    
                   
                    $stDadosFixos['DSARQXMLNFCE']             = $CFeCanc->asXML();
                    $chaveAcesso                              = $CFeCanc->infCFe->attributes()->chCanc ? strval($CFeCanc->infCFe->attributes()->chCanc) : strval($CFeCanc->infCFe->attributes()->Id);
                    $chaveAcessoCancelada                     = $CFeCanc->infCFe->attributes()->Id ? strval($CFeCanc->infCFe->attributes()->Id) : strval($CFeCanc->infCFe->attributes()->chCanc);
                    $arrayDadosAux['IDTIPOARQ']               = 'S';
                    $arrayDadosAux['error']['ERROSTATUS'][]   = 'F';
                    $arrayDadosAux['stNrOrg']                 = $stDadosFixos['NRORG']; 
                    $arrayDadosAux['error']['NMARQUIVOXML']   = $stDadosFixos['NMARQUIVOXML'];
                    $arrayDadosAux['error']['NRACESSONFCE']   = $chaveAcesso;
                    $arrayDadosAux['error']['NRACESSOCANC']   = $chaveAcessoCancelada;
                    $arrayDadosAux['error']['NRNOTAFISCALCE'][] = str_pad($CFeCanc->infCFe->ide->nCFe,  6, 0,STR_PAD_LEFT);
                    
                    $stData         = $CFeCanc->infCFe->ide->dEmi;
                    
                    $stDataVenda    = substr($stData,6,2).'/'.substr($stData,4,2).'/'.substr($stData,0,4);
                    
                    //TROCADO A CHAVE PELA CHAVE DA NOTA CANCELADA
                    //$params['NRACESSONFCE']       = $chaveAcesso;
                    $params['NRACESSONFCE']       = $chaveAcessoCancelada;
                    $params['DSNOMEARQUIVO']      = $stDadosFixos['NMARQUIVOXML'];           
                    $params['DSARQXMLNFCE']       = $stDadosFixos['DSARQXMLNFCE'];
                    $params['DTENTRVENDA']        = $stDataVenda;
                    $params['CDFILIAL']           = $stDadosFixos['CDFILIAL'];        
                    $params['CDCAIXA']            = $stDadosFixos['CDCAIXA'];  
                    $params['NRORG']              = $stDadosFixos['NRORG'];
                    $params['CDOPERADOR']         = $stDadosFixos['CDOPERADOR'];
                    $params['NRIMPARQXMLNFCE']    = NULL;
                    $params['IDSTATUSIMPARQ']     = 'N';
                    $params['IDTIPOARQ']          = $arrayDadosAux['IDTIPOARQ'];
                   //Insere os dados do arquivo no banco
                    try{
                        $arrayDadosAux['error']['IDSTATUSIMPARQ']  = 'N';
                        
                        $result = $this->expImpNfceService->insertLogArquivoXml($params,$connection);
                        
                        $arrayDadosAux['error']['NRIMPARQXMLNFCE'] = $result['P_NRIMPARQXMLNFCE'];
                        $arrayDadosAux['error']['IDSTATUSIMPARQ']  = $result['P_IDSTATUSIMPARQ'];
                        if($result['P_IDSTATUSIMPARQ'] == 'S'){
                            $arrayDadosAux = $this->buildRetorno($arrayDadosAux,'T','A','Nota já importada','','','','');
                            $arrayDados[] = $arrayDadosAux['error'];
                            unset($arrayDadosAux);
                            $connection->commit();
                            continue;
                        }
                    }
                    catch(\Exception $e){
                        $msg = $e->getMessage();
                        if(strpos($msg, 'ERRO_MESMO_INSTANTE') !== false){
                            $arrayDadosAux = $this->buildRetorno($arrayDadosAux,'T','E','Arquivo em processamento no momento. Tente novamente mais tarde.','','','','');
                            $arrayDados[] = $arrayDadosAux['error'];
                            unset($arrayDadosAux);
                            $connection->rollback();
                            continue;
                        }
                        else{
                            throw $e;
                        }
                    }
                    
                    $arrayDadosAux = $this->geraXmlSatCanc($CFeCanc,$arrayDadosAux,$stDadosFixos,$connection);
                    if(in_array('T',$arrayDadosAux['error']['ERROSTATUS'])){
                        $connection->rollback();
                        continue;
                    }
                     
                    $arrayDadosAux = $this->setVendaSatCanc($arrayDadosAux,$connection);
                    if(in_array('T',$arrayDadosAux['error']['ERROSTATUS'])){
                        $connection->rollback();
                    }
                    $arrayDados[] = $arrayDadosAux['error'];
                    unset($arrayDadosAux);
                }
                
                return $arrayDados;
                
            
            //Processamento de eventos de cancelamento
            }else if(count($xmlFile->retEvento)){
                //informações de envio do evento
                $infEvento = null;
                if($xmlFile->evento)
                    $infEvento = $xmlFile->evento->infEvento;
                    
                //informações de envio de 
                $infEventoRetorno = $xmlFile->retEvento->infEvento;
                
                $nrAcesso = (string)$infEventoRetorno->chNFe;
                $nrAcessoVenda = $nrAcesso;
                $tpEvento = (string)$infEventoRetorno->tpEvento;
                $tipoEventoAceito = true;
                
                $nrAcessoSubstituicao = null;
                //Evento de cancelamento
                if($tpEvento == '110111'){
                    $nrAcesso = 'NC' . $nrAcesso;    
                }
                
                //Evento de cancelamento por substituição
                else if ($tpEvento == '110112'){
                    $nrAcesso = 'NS' . $nrAcesso;
                    if($infEvento) {
                        if($infEvento->detEvento->chNFeRef){
                            $nrAcessoSubstituicao = (string) $infEvento->detEvento->chNFeRef; 
                        }
                    }
                }
                else{
                    $tipoEventoAceito = false;
                }
                
                $statusProcessamento = (string)$infEventoRetorno->cStat;
                $motivoRejeicao = "";
                if($statusProcessamento !== "135"){
                    if($infEventoRetorno->xMotivo){
                        $motivoRejeicao = (string)$infEventoRetorno->xMotivo;
                    }
                }
                
                //Utilizada data de registro de evento no sefaz
                $data  = (string)$infEventoRetorno->dhRegEvento;
                $data = date('d/m/Y', strtotime($data));
                
                //Dados para retorno ao frontend
                $arrayDados['isSat']               = false;
                $arrayDados['cdsVendaXml']         = ['NRNOTAFISCALCE' => ''];
                $arrayDados['cdsItensXml']         = NULL;
                $arrayDados['cdsMeioPgtoXml']      = NULL;
                $arrayDados['stFilial']            = $stDadosFixos['CDFILIAL'];
                $arrayDados['stCdCaixa']           = $stDadosFixos['CDCAIXA'];
                $arrayDados['stEmpresa']           = $stDadosFixos['CDEMPRESA'];
                $arrayDados['stTipoEmissao']       = 'N';
                $arrayDados['seFiles']             = $stDadosFixos['NMARQUIVOXML'];
                $arrayDados['stCdOperador']        = $stDadosFixos['CDOPERADOR'];
                $arrayDados['IDTIPOARQ']               = 'N';
                $arrayDados['error']['ERROSTATUS'][]   = 'F';
                $arrayDados['stNrOrg']                 = $stDadosFixos['NRORG']; 
                $arrayDados['error']['NMARQUIVOXML']   = $stDadosFixos['NMARQUIVOXML'];
                $arrayDados['error']['NRACESSONFCE']   = $nrAcessoVenda;
                $arrayDados['error']['NRNOTAFISCALCE'] = "";
                
                
                
                $params['NRACESSONFCE']       = $nrAcesso;
                $params['DSNOMEARQUIVO']      = $stDadosFixos['NMARQUIVOXML'];           
                $params['DSARQXMLNFCE']       = $xmlFile->asXML(); 
                $params['DTENTRVENDA']        = $data;     
                $params['CDFILIAL']           = $stDadosFixos['CDFILIAL'];        
                $params['CDCAIXA']            = $stDadosFixos['CDCAIXA'];  
                $params['NRORG']              = $stDadosFixos['NRORG'];
                $params['NRIMPARQXMLNFCE']    = NULL;
                $params['IDSTATUSIMPARQ']     = 'N';
                $params['IDTIPOARQ']          = $arrayDados['IDTIPOARQ'];
                $params['CDOPERADOR']         = $stDadosFixos['CDOPERADOR'];
                
                $connection->beginTransaction();
                
                //Insere os dados do arquivo no banco
                try{
                    $arrayDados['error']['IDSTATUSIMPARQ']  = 'N';
                    $result = $this->expImpNfceService->insertLogArquivoXml($params,$connection);
                    
                    $arrayDados['error']['NRIMPARQXMLNFCE'] = $result['P_NRIMPARQXMLNFCE'];
                    $arrayDados['error']['IDSTATUSIMPARQ']  = $result['P_IDSTATUSIMPARQ'];
                   
                    if($result['P_IDSTATUSIMPARQ'] == 'S'){
                        $arrayDados = $this->buildRetorno($arrayDados,'T','E','Evento já importado','','','','');
                        return $arrayDados['error'];
                    }
                }
                catch(\Exception $e){
                    $msg = $e->getMessage();
                    if(strpos($msg, 'ERRO_MESMO_INSTANTE') !== false){
                        $arrayDados = $this->buildRetorno($arrayDados,'T','E','Arquivo em processamento no momento. Tente novamente mais tarde.','','','','');
                        return $arrayDados['error'];
                    }
                    else{
                        throw $e;
                    }
                }
                
                $paramConsultaVenda = array();
                $paramConsultaVenda['NRACESSONFCE'] = $nrAcessoVenda;
                //Verifica venda existente pela chave de acesso
                $consultaVendaChaveAcesso = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIF_VENDA_NRACESSONFCE,$paramConsultaVenda,$connection)->getResult();
                
                //Se a venda não for encontrada
                if(empty($consultaVendaChaveAcesso)){
                    $arrayDados = $this->buildRetorno($arrayDados,'F','E','Venda com chave de acesso indicada não encontrada', '','','','');
                }
                //Se evento não é do tipo aceito
                if (!$tipoEventoAceito){
                    $arrayDados = $this->buildRetorno($arrayDados,'F','E','Evento indicado no arquivo não reconhecido.', '','','','');
                }
                //Se evento foi recusado pelo SEFAZ
                if($statusProcessamento !== "135"){
                    $arrayDados = $this->buildRetorno($arrayDados,'F','E','Cancelamento não concluído junto ao SEFAZ','','','','');
                }
                //Chave de acesso de substituição (se evento correspondente)
                if($nrAcessoSubstituicao){
                    $arrayDados = $this->buildRetorno($arrayDados,'F','A','Chave de acesso da nota para substituição: ' . $nrAcessoSubstituicao,'','','','');
                }
                //Motivo da rejeição
                if($motivoRejeicao){
                    $arrayDados = $this->buildRetorno($arrayDados,'F','E', $motivoRejeicao,'','','','');
                }
                $vendaCanceladaComSucesso = false;
                if(!empty($consultaVendaChaveAcesso)){
                    $venda = $consultaVendaChaveAcesso[0];
                    //Venda já está cancelada
                    if($venda['IDSITUVENDA'] == 'C'){
                        $arrayDados = $this->buildRetorno($arrayDados,'F','E','Venda com chave de acesso indicada já cancelada no sistema','','','','');
                    }
                    
                    if($tipoEventoAceito && $statusProcessamento == "135" && $venda['IDSITUVENDA'] !== 'C') {
                        
                        $dataVendaParaRetorno = \DateTime::createFromFormat('d/m/Y H:i:s', $venda['DTENTRVENDA']);
                        $dataVendaParaRetorno = $dataVendaParaRetorno->format('d/m/Y');
                        
                        
                        //Atualiza venda para cancelada
                        $paramsUpdatesCancelamento = array();
                        $paramsUpdatesCancelamento['CDFILIAL']       =  $venda['CDFILIAL'];
                        $paramsUpdatesCancelamento['CDCAIXA']        =  $venda['CDCAIXA'];
                        $paramsUpdatesCancelamento['NRSEQVENDA']     =  $venda['NRSEQVENDA'];
                        $paramsUpdatesCancelamento['NRACESSOCANC']   =  $nrAcessoVenda;
                        $paramsUpdatesCancelamento['DSARQXMLCANC']   =  $params['DSARQXMLNFCE'];
                
                        $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::UPDATE_VENDA_CANC,$paramsUpdatesCancelamento, $connection);
                        $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::UPDATE_MOV_CAIXA_SATCANC,$paramsUpdatesCancelamento, $connection);
                        $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::UPDATE_ARQXML_SATCANC,$paramsUpdatesCancelamento, $connection);
                        
                        $paramsStatusImp = array();
                        $paramsStatusImp['NRSEQVENDA'] = $venda['NRSEQVENDA'];
                        $paramsStatusImp['IDSTATUSIMPARQ'] = 'S';
                        $paramsStatusImp['NRACESSONFCE'] = $nrAcesso;
                        $arrayDados = $this->buildRetorno($arrayDados,'F','A','Venda cancelada Nro: ' . $venda['NRSEQVENDA'] . ' - ' . $dataVendaParaRetorno,'','','','');
                        //Indica que o arquivo foi importado com sucesso
                        $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::UPDATE_STATUS_IMP_ARQXML,$paramsStatusImp,$connection);
                        $vendaCanceladaComSucesso = true;
                        $connection->commit();
                    }
                }
                
                if(isset($arrayDados['error']['IDTIPOMSG'])){
                    if(count($arrayDados['error']['IDTIPOMSG'])){
                        //Modifica a chave de acesso para salvamento na tabela de mensagens
                        $arrayDadosForMsg = $arrayDados;
                        $arrayDadosForMsg['error']['NRACESSONFCE'] = $nrAcesso;
                        $this->expImpNfceService->insertMsgLogArquivoXml($arrayDadosForMsg,$connection);
                    }
                }
                if($vendaCanceladaComSucesso) {
                    $arrayDados = $this->buildRetorno($arrayDados,'F','S','Evento importado e venda cancelada com sucesso.','','','','');
                    $arrayDados['error']['IDSTATUSIMPARQ']   = 'S';
                }
                else{
                    $connection->rollback();
                }
                return $arrayDados['error'];
            }else if(count($xmlFile->inutNFe)){
                $params['NRINSJURFILI'] =  $row["CNPJ"]=(string)$xmlFile->retInutNFe->infInut->CNPJ;
                $row["DTVNDCARDS"]=substr((string)$xmlFile->retInutNFe->infInut->dhRecbto,0,10);
                $row["IDTPAMBNFCE"] = (string)$xmlFile->inutNFe->infInut->tpAmb;
                $row["SENFC"] = (string)$xmlFile->inutNFe->infInut->serie;
                $row["NMRF"] = (string)$xmlFile->inutNFe->infInut->nNFFin;
                $row["NMRI"] = (string)$xmlFile->inutNFe->infInut->nNFIni;
                $row["CDFILIAL"] = $stDadosFixos["CDFILIAL"];
                $row["CDCAIXA"] = $stDadosFixos["CDCAIXA"];
                $consultaFilialNfce  = $this->expImpNfceService->executarQueryInuti(ExpImpNfceService::QUERY_GET_FILIAL_INUTI,$params,$connection);
                if($row["CDFILIAL"]==$consultaFilialNfce["CDFILIAL"]){
                    $hoje = date('d/m/Y');
                    $DataInicial= date('d/m/Y', strtotime($row["DTVNDCARDS"]));
                    $row["NMRF"]=str_pad($row["NMRF"] , 9 , '0' , STR_PAD_LEFT);
                    $row["NMRI"]=str_pad($row["NMRI"] , 9 , '0' , STR_PAD_LEFT);
                    $row["SENFC"]=str_pad($row["SENFC"] , 3 , '0' , STR_PAD_LEFT);
                    $params = array(
                        "P_CDFILIAL" => $consultaFilialNfce["CDFILIAL"],
                        "P_CDCAIXA" => $row["CDCAIXA"],
                        "P_NRNOTAINICIAL" => $row["NMRI"],
                        "P_NRNOTAFINAL" => $row["NMRF"],
                        "P_CDSERIENFCE" => $row["SENFC"],
                        "P_IDTPAMBNFCE" => $row["IDTPAMBNFCE"],
                        "P_DTINICIAL" => $DataInicial,
                        "P_DTFINAL" => $hoje,
                        "P_DTENTRVENDA" => $DataInicial
                    );
                    $result = $this->expImpNfceService->executarQueryInsert(TransmissaoNFCE::INSERIR_NOTAS_INUTILIZADAS_IMPORT,$params,$connection);
                    $arrayDadosAux['error']['NMARQUIVOXML']     = $stDadosFixos['NMARQUIVOXML'];
                    $arrayDadosAux['error']['IDSTATUSIMPARQ']   = 'S';
                    $arrayDadosAux = $this->buildRetorno($arrayDadosAux,'S','S','Nota inutilizada inserida na tela de notas inutilizadas com sucesso.','','','','');
                    $arrayDados[] = $arrayDadosAux['error'];
                    return $arrayDados;
                   // $arrayDados = $this->buildRetorno($arrayDados,'F','S','Evento importado e venda cancelada com sucesso.','','','','');
                }
                else{
                    $arrayDadosAux['error']['NMARQUIVOXML']     = $stDadosFixos['NMARQUIVOXML'];
                    $arrayDadosAux['error']['IDSTATUSIMPARQ']   = 'N';
                    $arrayDadosAux = $this->buildRetorno($arrayDadosAux,'T','E','A filial selecionada e diferente da filial da nota inutilizada','Selecionar a mesma filial da nota','','','');
                    $arrayDados[] = $arrayDadosAux['error'];
                    return $arrayDados;
                }
                

            }else{
                $arrayDadosAux['error']['NMARQUIVOXML']     = $stDadosFixos['NMARQUIVOXML'];
                $arrayDadosAux['error']['IDSTATUSIMPARQ']   = 'N';
                $arrayDadosAux = $this->buildRetorno($arrayDadosAux,'T','E','Formato do arquivo incorreto','','','','');
                $arrayDados[] = $arrayDadosAux['error'];
                return $arrayDados;
            }
            
            
        }catch (\Exception $e){
            $connection->rollback();
            // adicionar retorno 
            throw new Exception($e);
             
        }
        
    }
    
    public function importarXml (Request\DataSet $request, Response $response) {
         
        
            
            $row = $request->getDataSet()->getRows()[0];
            
             
            $stDadosFixos['CDFILIAL']   = $row['CDFILIAL'];
            $stDadosFixos['CDCAIXA']    = $row['CDCAIXA'];
            $stDadosFixos['CDEMPRESA']  = $row['CDEMPRESA'];
            $stDadosFixos['CDOPERADOR'] = $row['CDOPERADOR'];
            
          
            
            //percorre a $row['ARQUIVOXML'] e executa cada arquivo recebido
            foreach ($row['ARQUIVOXML'] as $fileIndex=>$dados){
                
                
                 $b64 = explode(",", $dados['b64File'])[1];
                 $decodedB64 = base64_decode($b64);
                 $stDadosFixos['DSARQXMLNFCE'] = $decodedB64;
                 $stDadosFixos['NMARQUIVOXML'] = $dados['name'];
                 $stDadosFixos['NRORG']              = $row['NRORG'];
                 $stDadosFixos['CDOPERADOR']         = $row['CDOPERADOR'];
                   
                $arrayDados[$fileIndex] = $this->gravaArqXml($decodedB64,$stDadosFixos);
                
            }
            
           
         $response->addDataset(new DataSet('retorno', $arrayDados));
       
         
       
     }
     
    public function reimportarXml (Request\DataSet $request, Response $response) {
            
            $row = $request->getDataSet()->getRows()[0];
            
            if(!isset($row['CDEMPRESA'])){
                $params['CDFILIAL'] = $row['CDFILIAL'];
                $consultaFilialEmpresa = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_FILIAL_EMPRESA,$params)->getResult();
                $stDadosFixos['CDEMPRESA']  = $consultaFilialEmpresa[0]['CDEMPRESA']; 
            }else{
                $stDadosFixos['CDEMPRESA']  = $row['CDEMPRESA'];
            }
            $stDadosFixos['CDOPERADOR'] = $row['CDOPERADOR'];
            $stDadosFixos['NRORG']      = $row['NRORG'];
            
            //percorre a $row['ARQUIVOXML'] e executa cada arquivo recebido
            foreach ($row['ARQUIVOXML'] as $fileIndex=>$dados){
                $stDadosFixos['CDFILIAL']   = $dados['CDFILIAL'];
                $stDadosFixos['CDCAIXA']    = $dados['CDCAIXA'];
                
                $params['P_CDCAIXA']        = $dados['CDCAIXA'];
                $params['P_CDFILIAL']       = $dados['CDFILIAL'];
                $params['P_DSNOMEARQUIVO']  = $dados['DSNOMEARQUIVO'];
                $params['P_DTENTRVENDA']    = $dados['DTENTRVENDA'];
                
                
                $consultaArquivo  = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_GET_FILES,$params)->getResult();
               
                foreach($consultaArquivo as $notaIndex=>$nota){
                                     
                     if($nota['IDSTATUSIMPARQ'] == 'N'){
                         $arquivo = $nota['DSARQXMLNFCE'];
                         
                         $stDadosFixos['DSARQXMLNFCE'] = $arquivo;
                         $stDadosFixos['NMARQUIVOXML'] = $nota['DSNOMEARQUIVO'];
                        
                         $arrayDados[$fileIndex][$notaIndex] = $this->gravaArqXml($arquivo,$stDadosFixos);
                        
                     }else{
                        $arrayDadosAux[$fileIndex][$notaIndex]['error']['IDSTATUSIMPARQ']   = 'N';
                        $arrayDadosAux[$fileIndex][$notaIndex]['error']['NMARQUIVOXML']     = $nota['DSNOMEARQUIVO'];
                       
                        $arrayDadosAux[$fileIndex][$notaIndex]['error']['NRACESSONFCE']     = $nota['NRACESSONFCE'];
                        $arrayDadosAux[$fileIndex][$notaIndex] = $this->buildRetorno($arrayDadosAux[$fileIndex][$notaIndex],'T','A','Arquivo já importado','','','','');
                        $arrayDados[$fileIndex][$notaIndex][] = $arrayDadosAux[$fileIndex][$notaIndex]['error'];
                        unset($arrayDadosAux);
                     }
                }
            }
            
           
         $response->addDataset(new DataSet('retorno', $arrayDados));
       
         
       
     }
     
    private static function buildRetorno($arrayDados,$erroStatus,$idTipoMsg,$dsErro,$dsAjuda,$cdProduto,$nmProduto,$cdTiporece){
         
            $arrayDados['error']['ERROSTATUS'][]     = $erroStatus;
            $arrayDados['error']['IDTIPOMSG'][]    = $idTipoMsg;
            $arrayDados['error']['DSERRO'][]       = $dsErro;
            $arrayDados['error']['DSAJUDA'][]      = $dsAjuda;
            $arrayDados['error']['CDPRODUTO'][]      = $cdProduto;
            $arrayDados['error']['NMPRODUTO'][]      = $nmProduto;
            $arrayDados['error']['CDTIPORECE'][]      = $cdTiporece;
            
            return $arrayDados;
     }
    
    public static function escreverLog ($data) {
        array_push(self::$logMessages, $data);
    }
    
    public static function escreverLogArray($array) {
        foreach ($array as $linha) {
            self::escreverLog($linha);
        }
    }
    
    public static function getLog () {
        return self::$logMessages;
    }
    
    public function buscar (Request\Filter $request, Response $response) {
        ini_set('max_execution_time', 1800);
		ini_set('memory_limit', '1G');
        $parametros = $this->buildParameters($request);
        $valid = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VALID_QTD_VND, $parametros);
        
        try {
            if (($valid->getResult()[0]['COUNT(*)']) < 17000){
                $result = $this->expImpNfceService->executarQuery(ExpImpNfceService::BUSCAR_NOTAS, $parametros);
                if ($result->error())
                    throw new \Exception("Erro! ".$result->getMessage(), 1);
                else 
                    $response->addDataSet(new DataSet('vnd04400_expimpxmlnfce', $result->getResult()));
            } else {
                throw new \Exception("A quantidade de xml filtrado não é suportada pelo sistema. Filtre um período menor!");
            }
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    public function buildParameters (Request\Filter $request) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $params = array();
        $params[':ALLFIEDLS'] = 'T';
        foreach ($conditions as $filtro) {
            
            if (is_array($filtro['value'])) {
                
                // Especifico desse controller
                if ($filtro['columnName'] == "DTVENDA") {
                    if (!empty($filtro['value'][0]) && !empty($filtro['value'][1])) {
                        $params[':P_DTINI'] = $filtro['value'][0];
                        $params[':P_DTFIM'] = $filtro['value'][1];
                    }
                }
                
                if(strpos($filtro['columnName'], '|') && $filtro['operator'] === 'MAPPED_LIKE_ALL'){
                    $colunas = explode('|', $filtro['columnName']);
                    foreach($colunas as $k => $con){
                        $params[':ALLFIEDLS'] = $filtro['value'][0]['_ALL'];
                    }
                }else{
                    $params[':'.$filtro['columnName']] = implode("_", $filtro['value']);     
                }
            } else {
                $params[':'.$filtro['columnName']] = $filtro['value']; 
            }
        }
        
        // Específico desse controller
        $params = $this->verificarDatas($params);
        $params = $this->formataParametros($params);
        return $params;
    }
    
    public function verificarDatas($parametros) {
        
        if (empty($parametros[':P_DTINI']) || empty($parametros[':P_DTFIM'])) {
            $parametros[':P_DTINI'] = 
            $parametros[':P_DTFIM'] = date('d/m/Y');
        }
        unset($parametros[':DTVENDA']);
        return $parametros;
    }
    
    public function formataParametros($parametros){
        
        if(isset($parametros[':CDFILIAL'])){
            $parametros[':P_CDFILIAL'] = $parametros[':CDFILIAL'];
            unset($parametros[':CDFILIAL']);
        }else{
            $parametros[':P_CDFILIAL'] = '';
        }
        
        if($parametros[':P_CDFILIAL'] == ''){
            $parametros[':P_TCDFILIAL'] = 'T';
        }else{
            $parametros[':P_TCDFILIAL'] = '';
        }
        
        if(isset($parametros[':CDCAIXA'])){
            $parametros[':P_CDCAIXA'] = $parametros[':CDCAIXA'];
            unset($parametros[':CDCAIXA']);
        }else{
            $parametros[':P_CDCAIXA'] = '';
        }
        
        if($parametros[':P_CDCAIXA'] == ''){
            $parametros[':P_TCDCAIXA'] = 'T';
        }else{
            $parametros[':P_TCDCAIXA'] = '';
        }
        
        if(isset($parametros[':SITUACAO'])){
            $parametros[':P_IDSTATUSNFCE'] = $parametros[':SITUACAO'];
            unset($parametros[':SITUACAO']);
        }else{
            $parametros[':P_IDSTATUSNFCE'] = 'A_C';
        }
        
        if(isset($parametros[':NRNOTAFISCALCE'])){
            $parametros[':NRNOTAFISCALCE'] = str_pad($parametros[':NRNOTAFISCALCE'], 6, '0', STR_PAD_LEFT);
        }else{
            $parametros[':NRNOTAFISCALCE'] = 'T';
        }
        
        return $parametros;
    }
    
    public function diretorioXML(Request\Row $request, Response $response){
        $row = $request->getRow();
        $retorno = new Resposta();
        
        $arrayCurlFields = array(
            'diretorio' => date('Ymd').'-'.uniqid(rand(), false)
        );
        $response->addDataSet(new DataSet('resultado', $arrayCurlFields));
    }
    
    public function formatXML($notas, $HABDESTAG = 'N'){
        
        if(!empty($notas) && count($notas) > 0){
            
            foreach ($notas as $key => $value) {
                
                try{
                    if(!empty($value['DSARQXMLNFCE'])){
                        
                        $xml = str_replace('<?xml version="1.0" encoding="UTF-8"?>', '', $value['DSARQXMLNFCE']);
                        $xml = str_replace('<?xml version="1.0"?>', '', $xml);
                        
                        $verificaProtcXML = strpos( $xml, '<nfeProc');
                        $verificaXML = strpos( $xml, '<NFe');
                        
                        if($value['IDTPEMISVEND'] == 'N' && $verificaProtcXML !== false){
                            
                            $notas[$key]['DSARQXMLNFCE'] = $xml;
                            $notas[$key]['NOMEDSARQXMLNFCE'] = 'NFCe'.$value['NRACESSONFCE'].'-pro-rec.xml';
                            
                            if($value['IDSTATUSNFCE'] == 'C'){
                                $XCanc = $this->xmlCancelado($value);
                                $notas[$key]['DSARQXMLCANCNFCE'] = $XCanc;
                                $notas[$key]['NOMEDSARQXMLCANCNFCE'] = 'NFCe'.$value['NRACESSONFCE'].'-pro-can.xml';
                            }
                        }else if($value['IDTPEMISVEND'] == 'N' && $verificaProtcXML === false && $verificaXML !== false) {
                           
                            $tagPos = '';
                            $xmlRetorno = simplexml_load_string($xml);  
                            
                            if(!empty($xmlRetorno)){
                                
                                $arrayData2 = $this->xmlToArray($xmlRetorno);
                                //verifica se XML foi assinado
                                $verificaXMLSign = strpos($xml, 'Signature');
                                if($verificaXMLSign === false){
                                	$xml = $this->signNFCe($value, $xml);
                                	if(!$xml){
                                		//valida os dados do XML
                                		 if(!$this->validaXMLSalvo($arrayData2)){
                                			 //regera XML com os dados de venda atualizado
                                			 $xml = $this->callRemontaXML($value);
                                			 $xml = $this->signNFCe($value, $xml);
                                		 }
                                	}
                                }
                                 
                                $tagPos = '</infNFe>';
                                $xmlRetorno = simplexml_load_string($xml);  
                                $arrayData2 = $this->xmlToArray($xmlRetorno);
                                	
                                $arrayDataEnviNFe = array();
                                if(isset($arrayData2['enviNFe']) && $HABDESTAG == 'S'){
                                	$arrayDataEnviNFe['enviNFe'] = $arrayData2['enviNFe'];
                                	$arrayData['NFe'] = $arrayData2['enviNFe']['NFe'];
                                }else if(isset($arrayData2['enviNFe']) && $HABDESTAG == 'N'){
                                	$arrayDataEnviNFe['enviNFe'] = '';
                                	$arrayData['NFe'] = $arrayData2['enviNFe']['NFe'];
                                	
                                	//retira conforme orientacao do delphi => O xml deve ser gerado a partir da tag <NFe xmlns="http://www.portalfiscal.inf.br/nfe">
                                	$posIn = strpos($xml, '<NFe');
                                	$xmlGerado = substr($xml, $posIn);
                                	$posFi = strpos($xmlGerado, '</enviNFe>');
                                	$xmlGerado = substr($xmlGerado, 0, $posFi);
                                	$xml = $xmlGerado;
                                	
                                }else{
                                	$arrayData = $arrayData2;
                                }
                                
                                
                                if(!empty($value['DTHRPROTOCONFCE'])){
                                	$date = DateTime::createFromFormat('d/m/Y H:i:s', $value['DTHRPROTOCONFCE']);
                                	$dataFormatada1 = $date->format('Y-m-d');
                                	$dataFormatada2 = $date->format('H:i:s');
                                	$stFusoHr = '-03:00';
                                	$dhRecbto = $dataFormatada1."T".$dataFormatada2.$stFusoHr;    
                                }else{
                                    
                                	$dhRecbto = $arrayData['NFe']['infNFe']['ide']['dhEmi'];    
                                }
                                	
                                $stProtNfe = '<protNFe xmlns="http://www.portalfiscal.inf.br/nfe" versao="'.$arrayData['NFe']['infNFe']['@versao'].'">';
                                $stProtNfe .= '<infProt Id="ID'.$value['NRPROTOCOLONFCE'].'">';
                                $stProtNfe .= '<tpAmb>'.$value['IDTPAMBNFCE'].'</tpAmb>';
                                $stProtNfe .= '<verAplic>'.$arrayData['NFe']['infNFe']['@versao'].'</verAplic>';
                                $stProtNfe .= '<chNFe>'.$value['NRACESSONFCE'].'</chNFe>';
                                $stProtNfe .= '<dhRecbto>'.$dhRecbto.'</dhRecbto>';
                                $stProtNfe .= '<nProt>'.$value['NRPROTOCOLONFCE'].'</nProt>';
                                
                                $stProtNfe .= '<digVal>'.$arrayData['NFe']['Signature']['SignedInfo']['Reference']['DigestValue'].'</digVal>';
                                
                                
                                $stProtNfe .= '<cStat>'.$value['CODSTATUS'].'</cStat>';
                                $stProtNfe .= '<xMotivo>'.$value['DSOBSSTATUSNFCE'].'</xMotivo>';
                                $stProtNfe .= '</infProt></protNFe>';
                                
                                $stTextoXML = '';
                                $stTextoXML = '<?xml version="1.0" encoding="UTF-8"?><nfeProc versao="'.$arrayData['NFe']['infNFe']['@versao'].'" xmlns="http://www.portalfiscal.inf.br/nfe">';
                                $stTextoXML = $stTextoXML.$xml.$stProtNfe;
                                $stTextoXML = $stTextoXML.'</nfeProc>';
                                
                                
                                //mudancao pois trara na query principal
                                /*$parametros[':SGESTADO'] = $value['SGESTADO'];
                                $parametros[':IDSERVICO'] = ($value['IDTPEMISVEND'] == 'N') ? 'NFC' : '';
                                $parametros[':IDAMBIENTE'] = $value['IDTPAMBNFCE'];
                                
                                $urlChave = $this->expImpNfceService->executarQuery(ExpImpNfceService::BUSCAR_URLCHAVE, $parametros);
                                if(!empty($urlChave->getResult())){
                                	$url = $urlChave->getResult();
                                	$stUrlChave = '<urlChave>'.$url[0]['WSSITEUFQRCODENFC'].'</urlChave>';
                                }else{
                                	$stUrlChave = '<urlChave> </urlChave>';
                                }*/
                                
                                if(!empty($value['WSSITEUFQRCODENFC'])){
                                	$stUrlChave = '<urlChave>'.$value['WSSITEUFQRCODENFC'].'</urlChave>';
                                }else{
                                	$stUrlChave = '<urlChave> </urlChave>';
                                }
                                
                                if(strpos($stTextoXML, '<infNFeSupl>') == FALSE){
                                	$stQRCode = '<qrCode><![CDATA['.$value['DSQRCODENFCE'].']]></qrCode>';
                                	$stQRCODE = '<infNFeSupl>'.$stQRCode.$stUrlChave.'</infNFeSupl>';
                                	//$posIn = strripos($stTextoXML, $tagPos);
                                	$posFi = strpos($stTextoXML, '<Signature');
                                	$xmlQrCode = substr($stTextoXML, 0, $posFi). $stQRCODE. substr($stTextoXML, $posFi);
                                	$stTextoXML = $xmlQrCode;
                                }
                                
                                $notas[$key]['DSARQXMLNFCE'] = $stTextoXML;
                                $notas[$key]['NOMEDSARQXMLNFCE'] = 'NFCe'.$value['NRACESSONFCE'].'-pro-rec.xml';
                                
                                if($value['IDSTATUSNFCE'] == 'C'){
                                    $XCanc = $this->xmlCancelado($value);
                                    $notas[$key]['DSARQXMLCANCNFCE'] = $XCanc;
                                    $notas[$key]['NOMEDSARQXMLCANCNFCE'] = 'NFCe'.$value['NRACESSONFCE'].'-pro-can.xml';
                                }
                            }
                            
                        }else{
                            $notas[$key]['NOMEDSARQXMLNFCE'] = 'AD'.$value['NRACESSONFCE'].'.xml';
                        	if($value['IDSTATUSNFCE'] == 'C'){
                        		$notas[$key]['NOMEDSARQXMLCANCNFCE'] = 'ADC'.$value['NRACESSONFCE'].'.xml';
                        	}
                        }
                    }
                    
                }catch(\Exception $e){  
                    $notas['error'][] = array('message'   => str_pad($value['NRNOTAFISCALCE'],6,0,STR_PAD_LEFT),
                                              'exception' => $e->getMessage());
                    continue;
                    // throw new Exception('Erro ao gerar nota: '.str_pad($value['NRNOTAFISCALCE'],6,0,STR_PAD_LEFT).'<br><br>'.$e->getMessage(),0,$e);
                } 
            }    
        }
        return $notas;
    }
    
    private function xmlCancelado($value){
        
    	$xmlCancSalvo = $value['DSARQXMLCANCNFCE'];
    	$xmlCancelamentoSalvo = simplexml_load_string($xmlCancSalvo);
    	
    	$xmlCanc = '<procEventoNFe></procEventoNFe>';
    	$xmlCancelamento = simplexml_load_string($xmlCanc);
    	
    	$xmlCancelamento->addAttribute('versao', '1.00');
    	$xmlCancelamento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
    	
    	if(!empty($value['DTHRPROTOCOCANC'])){
    		$dateCan = DateTime::createFromFormat('d/m/Y H:i:s', $value['DTHRPROTOCOCANC']);
    		$dataFormatada1Can = $dateCan->format('Y-m-d');
    		$dataFormatada2Can = $dateCan->format('H:i:s');
    		$stFusoHrCan = '-03:00';
    		$dhRecbtoCan = $dataFormatada1Can."T".$dataFormatada2Can.$stFusoHrCan;    
    	}else{
    		$dataFormatada1Can = date('Y-m-d');
    		$dataFormatada2Can = date('H:i:s');
    		$stFusoHrCan = '-03:00';
    		$dhRecbtoCan = $dataFormatada1Can."T".$dataFormatada2Can.$stFusoHrCan;    
    	}
    	
    	if(isset($xmlCancelamentoSalvo->evento)){
    		$evento = $xmlCancelamento->addChild('evento');
    		$evento->addAttribute('versao', '1.00');
    		$evento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
    		if($xmlCancelamentoSalvo->evento->infEvento){
    			$infEvento = $evento->addChild('infEvento');
    			
    			$arrayID = $xmlCancelamentoSalvo->evento->infEvento->attributes()->Id;
    			foreach((array)$arrayID as $key => $val){
    				$id = $val;
    			}
    			
    			$infEvento->addAttribute('Id', $id);
    			$infEvento->addChild('cOrgao',$xmlCancelamentoSalvo->evento->infEvento->cOrgao);
    			$infEvento->addChild('tpAmb',$xmlCancelamentoSalvo->evento->infEvento->tpAmb);
    			$infEvento->addChild('CNPJ',$xmlCancelamentoSalvo->evento->infEvento->CNPJ);
    			$infEvento->addChild('chNFe',$xmlCancelamentoSalvo->evento->infEvento->chNFe);
    			$infEvento->addChild('dhEvento',$xmlCancelamentoSalvo->evento->infEvento->dhEvento);
    			$infEvento->addChild('tpEvento',$xmlCancelamentoSalvo->evento->infEvento->tpEvento);
    			$infEvento->addChild('nSeqEvento',$xmlCancelamentoSalvo->evento->infEvento->nSeqEvento);
    			$infEvento->addChild('verEvento',$xmlCancelamentoSalvo->evento->infEvento->verEvento);
    			
    			if($xmlCancelamentoSalvo->evento->infEvento->detEvento){
    				$detEvento = $infEvento->addChild('detEvento');
    				$arrayID = $xmlCancelamentoSalvo->evento->infEvento->detEvento->attributes()->versao;
    				foreach((array)$arrayID as $key => $val2){
    					$versao = $val2;
    				}
    				$detEvento->addAttribute('versao', $versao);
    				$detEvento->addChild('descEvento',$xmlCancelamentoSalvo->evento->infEvento->detEvento->descEvento);
    				$detEvento->addChild('nProt',$xmlCancelamentoSalvo->evento->infEvento->detEvento->nProt);
    				$detEvento->addChild('xJust',$xmlCancelamentoSalvo->evento->infEvento->detEvento->xJust);    
    			}else{
    				$detEvento = $infEvento->addChild('detEvento');
    				$detEvento->addAttribute('versao', '1.00');
    				$detEvento->addChild('descEvento','Cancelamento');
    				$detEvento->addChild('nProt',$value['NRPROTOCOLONFCE']);
    				$detEvento->addChild('xJust',$value['DSRAZAOCANCNFCE']);    
    			}
    		}else{
    			$infEvento = $evento->addChild('infEvento');
    			$infEvento->addAttribute('Id', 'ID'.'110111'.$value['NRACESSONFCE'].'01');
    			$infEvento->addChild('cOrgao',$value['CDESTADOIBGE']);
    			$infEvento->addChild('tpAmb',$value['IDTPAMBNFCE']);
    			$infEvento->addChild('CNPJ',$value['NRINSJURIEST']);
    			$infEvento->addChild('chNFe',$value['NRACESSONFCE']);
    			
    			$infEvento->addChild('dhEvento',$dhRecbtoCan);
    			$infEvento->addChild('tpEvento','110111');
    			$infEvento->addChild('nSeqEvento','1');
    			$infEvento->addChild('verEvento','1.00');
    			
    			$detEvento = $infEvento->addChild('detEvento');
    			$detEvento->addAttribute('versao', '1.00');
    			$detEvento->addChild('descEvento','Cancelamento');
    			$detEvento->addChild('nProt',$value['NRPROTOCOLONFCE']);
    			$detEvento->addChild('xJust',$value['DSRAZAOCANCNFCE']);
    		}
    	}else{
    		$evento = $xmlCancelamento->addChild('evento');
    		$evento->addAttribute('versao', '1.00');
    		$evento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
    		
    		$infEvento = $evento->addChild('infEvento');
    		$infEvento->addAttribute('Id', 'ID'.'110111'.$value['NRACESSONFCE'].'01');
    		$infEvento->addChild('cOrgao',$value['CDESTADOIBGE']);
    		$infEvento->addChild('tpAmb',$value['IDTPAMBNFCE']);
    		$infEvento->addChild('CNPJ',$value['NRINSJURIEST']);
    		$infEvento->addChild('chNFe',$value['NRACESSONFCE']);
    		
    		$infEvento->addChild('dhEvento',$dhRecbtoCan);
    		$infEvento->addChild('tpEvento','110111');
    		$infEvento->addChild('nSeqEvento','1');
    		$infEvento->addChild('verEvento','1.00');
    		
    		$detEvento = $infEvento->addChild('detEvento');
    		$detEvento->addAttribute('versao', '1.00');
    		$detEvento->addChild('descEvento','Cancelamento');
    		$detEvento->addChild('nProt',$value['NRPROTOCOLONFCE']);
    		$detEvento->addChild('xJust',$value['DSRAZAOCANCNFCE']);
    	}
    	
    	if(isset($xmlCancelamentoSalvo->retEnvEvento)){
    		if(isset($xmlCancelamentoSalvo->retEnvEvento->retEvento)){
    			$retEvento = $xmlCancelamento->addChild('retEvento');
    			$retEvento->addAttribute('versao', '1.00');
    			$retEvento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
    			if(isset($xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento)){
    				$infEventoRet = $retEvento->addChild('infEvento');
    				$infEventoRet->addChild('tpAmb',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->tpAmb);
    				$infEventoRet->addChild('verAplic',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->verAplic);
    				$infEventoRet->addChild('cOrgao',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->cOrgao);
    				$infEventoRet->addChild('cStat',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->cStat);
    				$infEventoRet->addChild('xMotivo',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->xMotivo);
    				$infEventoRet->addChild('chNFe',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->chNFe); 
    				$infEventoRet->addChild('tpEvento',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->tpEvento);
    				$infEventoRet->addChild('xEvento',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->xEvento);
    				$infEventoRet->addChild('nSeqEvento',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->nSeqEvento);
    				$infEventoRet->addChild('dhRegEvento',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->dhRegEvento);
    				$infEventoRet->addChild('nProt',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->nProt);
    			}else{
    				$infEventoRet = $retEvento->addChild('infEvento');
    				$infEventoRet->addChild('tpAmb',$value['IDTPAMBNFCE']);
    				$infEventoRet->addChild('verAplic','SEFAZ');
    				$infEventoRet->addChild('cOrgao',$value['CDESTADOIBGE']);
    				$infEventoRet->addChild('cStat','135');
    				$infEventoRet->addChild('xMotivo','Evento registrado e vinculado a NF-e');
    				$infEventoRet->addChild('chNFe',$value['NRACESSONFCE']);
    				$infEventoRet->addChild('tpEvento','110111');
    				$infEventoRet->addChild('xEvento','Cancelamento');
    				$infEventoRet->addChild('nSeqEvento','1');
    				$infEventoRet->addChild('dhRegEvento',$dhRecbtoCan);
    				$infEventoRet->addChild('nProt',$value['NRPROTOCOLOCANC']);
    			}
    		}else{
    			$retEvento = $xmlCancelamento->addChild('retEvento');
    			$retEvento->addAttribute('versao', '1.00');
    			$retEvento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
    			$infEventoRet = $retEvento->addChild('infEvento');
    			$infEventoRet->addChild('tpAmb',$value['IDTPAMBNFCE']);
    			$infEventoRet->addChild('verAplic','SEFAZ');
    			$infEventoRet->addChild('cOrgao',$value['CDESTADOIBGE']);
    			$infEventoRet->addChild('cStat','135');
    			$infEventoRet->addChild('xMotivo','Evento registrado e vinculado a NF-e');
    			$infEventoRet->addChild('chNFe',$value['NRACESSONFCE']);
    			$infEventoRet->addChild('tpEvento','110111');
    			$infEventoRet->addChild('xEvento','Cancelamento');
    			$infEventoRet->addChild('nSeqEvento','1');
    			$infEventoRet->addChild('dhRegEvento',$dhRecbtoCan);
    			$infEventoRet->addChild('nProt',$value['NRPROTOCOLOCANC']);
    		}
    	}else{
    		$retEvento = $xmlCancelamento->addChild('retEvento');
    		$retEvento->addAttribute('versao', '1.00');
    		$retEvento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
    		$infEventoRet = $retEvento->addChild('infEvento');
    		$infEventoRet->addChild('tpAmb',$value['IDTPAMBNFCE']);
    		$infEventoRet->addChild('verAplic','SEFAZ');
    		$infEventoRet->addChild('cOrgao',$value['CDESTADOIBGE']);
    		$infEventoRet->addChild('cStat','135');
    		$infEventoRet->addChild('xMotivo','Evento registrado e vinculado a NF-e');
    		$infEventoRet->addChild('chNFe',$value['NRACESSONFCE']);
    		$infEventoRet->addChild('tpEvento','110111');
    		$infEventoRet->addChild('xEvento','Cancelamento');
    		$infEventoRet->addChild('nSeqEvento','1');
    		$infEventoRet->addChild('dhRegEvento',$dhRecbtoCan);
    		$infEventoRet->addChild('nProt',$value['NRPROTOCOLOCANC']);
    	}
    	$XCanc = $xmlCancelamento->asXML();
    	return $XCanc;
    }
    
    public function exportarXml(Request\Row $request, Response $response){
       
            $row = $request->getRow();
            $retorno = new Resposta();
            //tratamento para habilitar ou desabilitar tags <idLote> e <indSinc>
            if(!isset($row['HABDESTAG'])){
                $row['HABDESTAG'] = 'N';
            }
            
            //realizar tratamento dos arquivos XMLs
            $notas = $this->formatXML($row['arrRows'], $row['HABDESTAG']);
            
            $arrayCurlFields = array(
                'script' => 'EXP',
                'diretorio' => $row['diretorio'],
                'rows' => $notas
            );
            
            $fields_string = http_build_query($arrayCurlFields);
            try {   
                if(!empty($fields_string)){ 
                   
                
                    $ch = curl_init();
                    curl_setopt( $ch, CURLOPT_URL, $this->url );
                    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                    curl_setopt($ch,CURLOPT_POST, 1);
                    curl_setopt($ch,CURLOPT_POSTFIELDS, $fields_string);
                    $retorno = json_decode(curl_exec( $ch ));
                    $array = (array) $retorno;
                    $array['errors'] = !empty($notas['error']) ? $notas['error'] : '';
                    curl_close( $ch );
                    
                    $response->addDataSet(new DataSet('resultado', $array));
                
                }
                  
            }catch(\Exception $e){  
                 
                throw new Exception("URL inválida: ".$e->getMessage(),0,$e);
            }  
        
    }
    
    public function compactarXML(Request\Row $request, Response $response){
        $row = $request->getRow();
        $retorno = new Resposta();
        
        $arrayCurlFields = array( 
            'diretorio' => $row['diretorio'],
            'script' => 'COMP'
        );
        
        try {
            
            $fields_string = http_build_query($arrayCurlFields);
            $ch = curl_init();
            curl_setopt( $ch, CURLOPT_URL, $this->url );
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $fields_string);
            $retorno = json_decode(curl_exec( $ch ));
            $array = (array) $retorno;
            
            curl_close( $ch );
            
            //Verifica se o link de download é valido
            $curl = curl_init();
            curl_setopt ($curl, CURLOPT_URL, $retorno->linkDownload);
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
            $result = curl_exec ($curl);
            curl_close ($curl);
             
            if(strpos($result,'"error":true')){
                $array['errors'] = array('message'   => "Nenhum arquivo gerado!",
                                              'exception' => '');
                
            }
            
        }
        catch(\Exception $e){
            throw new Exception("URL inválida: ",0,$e);
        }
      
        $response->addDataSet(new DataSet('resultado', $array));
    }
    
    public function deleteArquivosXML(Request\Row $request, Response $response){
        $row = $request->getRow();
        $retorno = new Resposta();
        
        $arrayCurlFields = array( 
            'diretorio' => $row['diretorio'].'.zip',
            'script' => 'DEL'
        );
        
        try {
            
            $fields_string = http_build_query($arrayCurlFields);
            $ch = curl_init();
            curl_setopt( $ch, CURLOPT_URL, $this->url );
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $fields_string);
            $retorno = json_decode(curl_exec( $ch ));
            $array = (array) $retorno;
            curl_close( $ch ); 
        }
        catch(\Exception $e){
            throw new Exception("URL inválida",0,$e);
        }
        
        $response->addDataSet(new DataSet('resultado', $array));
    }
    
    private function xmlToArray($xml, $options = array()) {
        $defaults = array(
            'namespaceSeparator' => ':', // você pode querer que isso seja algo diferente de um cólon
            'attributePrefix' => '@',    // para distinguir entre os nós e os atributos com o mesmo nome
            'alwaysArray' => array(),    // array de tags que devem sempre ser array
            'autoArray' => true,         // só criar arrays para as tags que aparecem mais de uma vez
            'textContent' => '$',        // chave utilizada para o conteúdo do texto de elementos
            'autoText' => true,          // pular chave "textContent" se o nó não tem atributos ou nós filho
            'keySearch' => false,        // pesquisa opcional e substituir na tag e nomes de atributos
            'keyReplace' => false        // substituir valores por valores acima de busca
        );
        $options = array_merge($defaults, $options);
        $namespaces = $xml->getDocNamespaces();
        $namespaces[''] = null; // adiciona namespace base(vazio) 
    
        // Obtém os atributos de todos os namespaces
        $attributesArray = array();
        foreach ($namespaces as $prefix => $namespace) {
            foreach ($xml->attributes($namespace) as $attributeName => $attribute) {
                // Substituir caracteres no nome do atributo
                if ($options['keySearch']) $attributeName =
                        str_replace($options['keySearch'], $options['keyReplace'], $attributeName);
                $attributeKey = $options['attributePrefix']
                        . ($prefix ? $prefix . $options['namespaceSeparator'] : '')
                        . $attributeName;
                $attributesArray[$attributeKey] = (string)$attribute;
            }
        }
    
        // Obtém nós filhos de todos os namespaces
        $tagsArray = array();
        foreach ($namespaces as $prefix => $namespace) {
            foreach ($xml->children($namespace) as $childXml) {
                // Recursividade em nós filho
                $childArray = $this->xmlToArray($childXml, $options);
                reset($childArray);
                list($childTagName, $childProperties) = $this->myEach($childArray);
                // Substituir caracteres no nome da tag
                if ($options['keySearch']) $childTagName =
                        str_replace($options['keySearch'], $options['keyReplace'], $childTagName);
                // Adiciona um prefixo namespace, se houver
                if ($prefix) $childTagName = $prefix . $options['namespaceSeparator'] . $childTagName;
    
                if (!isset($tagsArray[$childTagName])) {
                    // Só entra com esta chave
                    // Testa se as tags deste tipo deve ser sempre matrizes, não importa a contagem de elementos
                    $tagsArray[$childTagName] =
                            in_array($childTagName, $options['alwaysArray']) || !$options['autoArray']
                            ? array($childProperties) : $childProperties;
                } elseif (
                    is_array($tagsArray[$childTagName]) && array_keys($tagsArray[$childTagName])
                    === range(0, count($tagsArray[$childTagName]) - 1)
                ) {
                    $tagsArray[$childTagName][] = $childProperties;
                } else {
                    $tagsArray[$childTagName] = array($tagsArray[$childTagName], $childProperties);
                }
            }
        }
    
        // Obtém o texto do nó
        $textContentArray = array();
        $plainText = trim((string)$xml);
        if ($plainText !== '') $textContentArray[$options['textContent']] = $plainText;
    
        $propertiesArray = !$options['autoText'] || $attributesArray || $tagsArray || ($plainText === '')
                ? array_merge($attributesArray, $tagsArray, $textContentArray) : $plainText;
    
        // Retorna o nó como array
        return array(
            $xml->getName() => $propertiesArray
        );
    }
    
    private function myEach(&$arr) {
        $key = key($arr);
        $result = ($key === null) ? false : [$key, current($arr), 'key' => $key, 'value' => current($arr)];
        next($arr);
        return $result;
    }
    
    private function signNFCe($nota, $unsignedXML){
        
		//le certificado
		$certificado = $this->getCertificado($nota);
		//config certificado
		$this->nfeTools = $this->transmissaoService->getNfeTools($nota, $certificado);
		
    	try {
    		
    		$XmlSign = $this->nfeTools->signNFe($unsignedXML);
		    if(!empty($XmlSign)){
				return $XmlSign;
		    }else{
		    	return false;
		    }
			
		} catch (\Exception $e) {
			return false;
		}
    }
    
	
	private function getCertificado($nota){
		
		$params[':CDINSCESTA'] = $nota['CDINSCESTA'];
		$params[':CDFILIAL'] = $nota['CDFILIAL'];
		
		$result = $this->expImpNfceService->executarQuery(TransmissaoNFCE::GET_LINK_CERTIFICADO, $params);
		if($result->error()) {
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        } else {
            $result = $result->getResult();
        }
        
		if (sizeof($result) > 0) {
			if (empty($result[0]['CERTIFICADO'])) {
				throw new \Exception('Erro, certificado não parametrizado. Verifique os parâmetros nas seguintes telas: Empresa -> Inscrição Estadual, e Parâmetros da Unidade -> Frente de Caixa -> Definições NFC-e / SAT!');
			}
			$link = $result[0]['DSDIRCERTIFICADO'];
			
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, $link);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_TIMEOUT, 5);
			
			$resultLink = curl_exec($ch);
            $err    = curl_errno($ch);
            $errmsg = curl_error($ch);
			curl_close($ch);
			
			if ($err){
                throw new \Exception("Erro ao obter conteúdo do Certificado Digital: " . $errmsg);
            } 
            
            if (strpos($resultLink, 'Not Found')) {
            	throw new \Exception("Erro 404. Não foi possível recuperar o certificado no servidor.". PHP_EOL . 
            	"Verificar se o Link do Servidor de Imagens encontra-se parametrizado em Paramêtros Gerais => Gerais => Geral." . PHP_EOL . 
            	"Verificar se o certificado foi parametrizado em Parâmetros da Unidade => Frente de Caixa => Definições NFC-e/SAT e Empresa => Inscrição Estadual" . PHP_EOL .$errmsg);
            }
            
		} else {
			throw new \Exception('Não foi possível recuperar os dados de Inscrição Estadual parametrizada.' . PHP_EOL . 
            	'Verificar se o certificado foi parametrizado em Parâmetros da Unidade => Frente de Caixa => Definições NFC0e/SAT e Empresa => Inscrição Estadual');
		}
		return $result;
	}
    
    private function validaXMLSalvo($arrayXML){
        
        $valorControle = true;
        foreach($arrayXML as $key => $value){
            //tag NFe
            if(!isset($value['NFe'])){
                $valorControle = false;
            }
            
            //tag infNFe
            if(!isset($value['NFe']['infNFe'])){
                $valorControle = false;
            }
            
            //tag versao
            if(!empty($value['NFe']['infNFe']['@versao'])){
                 $valorControle = false;
            }
            
            //tag IDE
            $ide = $value['infNFe']['ide'];
            if(!empty($ide)){
                foreach($ide as $k => $v){
                     if(!empty($v['cUF'])){
                         $valorControle = false;
                     }
                     if(!empty($v['cUF'])){
                         $valorControle = false;
                     }
                     if(!empty($v['serie'])){
                         $valorControle = false;
                     }
                     if(!empty($v['nNF'])){
                         $valorControle = false;
                     }
                     if(!empty($v['cMunFG'])){
                         $valorControle = false;
                     }
                     if(!empty($v['cDV'])){
                         $valorControle = false;
                     }
                     if(!empty($v['tpAmb'])){
                         $valorControle = false;
                     }
                }   
            }else{
                 $valorControle = false;
            }
            
            //tag IDE
            $emit = $value['infNFe']['emit'];
            if(!empty($emit)){
                foreach($emit as $k => $v){
                     if(!empty($v['CNPJ'])){
                         $valorControle = false;
                     }
                     if(!empty($v['xNome'])){
                         $valorControle = false;
                     }
                     if(!empty($v['xFant'])){
                         $valorControle = false;
                     }
                     if(!empty($v['IE'])){
                         $valorControle = false;
                     }
                     if(!empty($v['CNAE'])){
                         $valorControle = false;
                     }
                     if(!empty($v['CRT'])){
                         $valorControle = false;
                     }
                }   
            }else{
                 $valorControle = false;
            }
        }
        
        return $valorControle;
        
    }
    
    private function callRemontaXML($nota){
	    
        $parametros = array(
            ':CDFILIAL' => $nota['CDFILIAL'],
            ':CDCAIXA' => $nota['CDCAIXA'],
            ':CDLOJA' => $nota['CDLOJA'],
            ':NRSEQVENDA'    => $nota['NRSEQVENDA']
        );
        
        $result = $this->expImpNfceService->executarQuery(TransmissaoNFCE::ADQTRANSMISSAONFCE, $parametros);
        if($result->error()) {
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        } else {
            if(!empty($result->getResult())){
               $resposta = $result->getResult();
               $resposta[0]['CDOPERADOR'] = null;
	    	   $resposta[0]['TOTALVENDA'] = $nota['VRTOTALVENDA'];
	    	   $resposta[0]['DTABERTUR'] = $nota['DTABERTUR'];
               $retornoXML = $this->transmissaoService->montaXMLTransmitir($resposta, false);
               
               if(!$retornoXML['error']){
                   return $retornoXML['xml'];
               }
            }
        }
    }
    
    public function apixml(Request\Row $request, Response $response) {
    
       $rows = $request->getRow();
       
    
        $row = array(
       'CDFILIAL'=>$rows["unidade"],
       'NRORG'=>$rows["nrorg"],
       'CNPJ'=>$rows["cnpj"],
       'CDCAIXA'=>$rows["caixa"],
       'NMARQUIVOXML'=>$rows["arquivo"],
       'DSARQUIVOXML'=>$rows["conteudo"],
       'CDEMPRESA'=>[null]
       
       );
       
    
       
      
       
       $connectionCustom =InstanceManager::getInstance()->getService('connectionCustom');
      
       
       $entityManager = $connectionCustom->updateVpdConnection($row['NRORG']);
       $entityManager = $entityManager ? $entityManager : InstanceManager::getInstance()->getService('entityManager');
       $connection = $entityManager->getConnection();
       $params = array( 'CDFILIAL' => $row['CDFILIAL'], 'CDCAIXA' => $row['CDCAIXA'] );
       $result = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_FILIAL_EMPRESA,$params,$connection)->getResult();
        
        
       //validacoes de unidade, cnpj e caixa
       if($result){
           
           //valida se o cnpj informado é o mesmo da unidade no banco de dados
           if($result[0]['NRINSJURFILI'] !== $row['CNPJ']){
                $resultado = [    "status"=>'Não importado', "errosAlertas"=>["tipo"=>'Erro', "Descricao"=>'CNPJ informado não é o mesmo cadastrado para a unidade informada']];
                return $response->addDataset(new DataSet('response', $resultado));
           }
           
           //Valida de o caixa informado existe
           $resultCaixa = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIF_CAIXA_API,$params,$connection)->getResult();
           if(empty($resultCaixa)){
                $resultado = ["status"=>'Não importado', "errosAlertas"=>["tipo"=>'Erro', "Descricao"=>'Caixa informado não existe na unidade informada']];
                return $response->addDataset(new DataSet('response', $resultado));
           }
        
            $row['CDEMPRESA']           = $result[0]["CDEMPRESA"];
            $stDadosFixos['CDFILIAL']   = $row['CDFILIAL'];
            $stDadosFixos['CDCAIXA']    = $row['CDCAIXA'];
            $stDadosFixos['CDEMPRESA']  = $row['CDEMPRESA'];
          
            $decodedB64 = base64_decode($row['DSARQUIVOXML']);
            $stDadosFixos['DSARQXMLNFCE'] = $decodedB64;
            $stDadosFixos['NMARQUIVOXML'] = $row['NMARQUIVOXML'];
            $stDadosFixos['NRORG']  = $row['NRORG'];
            $stDadosFixos['CDOPERADOR'] = null;
            
            $arrayDados = $this->gravaArqXml($decodedB64,$stDadosFixos,$connection);
               
            
            $resultado =  $this->buildResponseApi($arrayDados);
            
        
            return $response->addDataset(new DataSet('response', $resultado));
            
       }else{
            $resultado = [    "status"=>'Não importado', "errosAlertas"=>["tipo"=>'Erro', "Descricao"=>'Unidade Não encontrada']];
                                        
            return $response->addDataset(new DataSet('response', $resultado));
       }
    }
    
    const DEFAULT_XML = '
        <NFe xmlns="http://www.portalfiscal.inf.br/nfe">
        </NFe>';
        
    public function buildResponseApi($arrayDados){
            $status = 'Importado';  
              
            if(isset($arrayDados[0])){
                foreach($arrayDados as $dados){
                    if($dados["IDSTATUSIMPARQ"] == 'N') $status = 'Não importado';
                    $erros[] = $this->buildResponseMessage($dados);
                }
            }else{
                if($arrayDados["IDSTATUSIMPARQ"] == 'N') $status = 'Não importado';
                $erros = $this->buildResponseMessage($arrayDados);
            }
          
          $resultado = ["status" => $status, "errosAlertas" => $erros];
          
          return $resultado;
    }
    
    private function buildResponseMessage ($dados){
        for($i = 0; $i<count($dados["DSERRO"]); $i++){
                          
            $erro["tipo"]             = ($dados["IDTIPOMSG"][$i] == "E") ? "erro" : "alerta";
            $erro["descricao"]        = $dados["DSERRO"][$i];
            $erro["notaFiscal"]       = isset($dados["NRNOTAFISCALCE"][$i]) ? $dados["NRNOTAFISCALCE"][$i] : '';
            $erro["codigoProduto"]    = $dados["CDPRODUTO"][$i];
            $erro["nomeProduto"]      = $dados["NMPRODUTO"][$i];
            $erro["codigoPagamento"]  = $dados["CDTIPORECE"][$i];
            $erro["ajuda"]            = $dados["DSAJUDA"][$i];
            $erro['status']           = $dados["IDSTATUSIMPARQ"] == 'N' ? 'Não importado' : 'Importado';
        }
        return $erro;
    }
    
}